# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from ..scraper import Scraper


class Removed(Scraper):
    REASONS = {
        'jsh': 'Site is very JavaScript-heavy, writing a module would be' +
               ' very complicated.',
        'del': 'Comic was removed from the web.',
        'block': 'The comic site is blocking us.',
        'deny': "The comic doesn't want to be listed in Dosage (request from owner or robots.txt",
        'unk': 'Comic was removed for an unknown reason.',
        'brk': 'Comic navigation is broken.',
        'mov': 'Comic moved to a new hoster and no new module was written.',
        'mis': 'Pages are missing from the comic.',
        'acc': 'Account is needed to access site.',
        'legal': 'Comic was removed for legal reasons (fanrip, scanlation, etc.).',
    }

    def __init__(self, name, reason='del'):
        super(Removed, self).__init__(name)
        self.reason = reason

    def getDisabledReasons(self):
        return {'rem-' + self.reason: self.REASONS[self.reason]}

    @classmethod
    def getmodules(cls):
        return (
            # Removed in 2.16
            cls('AbleAndBaker'),
            cls('AlsoBagels'),
            cls('Antics'),
            cls('Arcamax/BleekerTheRechargeableDog'),
            cls('Arcamax/DeFlocked'),
            cls('Arcamax/TinasGroove'),
            cls('ASkeweredParadise'),
            cls('ASofterWorld', 'block'),
            cls('BackwaterPlanet'),
            cls('BigFatWhale'),
            cls('BizarreUprising'),
            cls('Blip'),
            cls('BoxerHockey'),
            cls('BoyOnAStickAndSlither', 'jsh'),
            cls('BrentalFloss'),
            cls('BrentalFloss/FlossedInTime'),
            cls('BrentalFloss/GuestComics'),
            cls('BrightlyWound'),
            cls('Caggage'),
            cls('Carciphona', 'jsh'),
            cls('Champ2010'),
            cls('CheckerboardNightmare'),
            # Patreon & Pixiv (https://www.patreon.com/Collar6)
            cls('Collar6', 'mov'),
            cls('ComicFury/30years'),
            cls('ComicFury/AAB'),
            cls('ComicFury/AdventuresofMaggie'),
            cls('ComicFury/Aether'),
            cls('ComicFury/Afairtrade'),
            cls('ComicFury/Afrodays'),
            cls('ComicFury/Albinobros'),
            cls('ComicFury/Alexanderandlucas'),
            cls('ComicFury/Alittlebitofeverything'),
            cls('ComicFury/Americanextremists'),
            cls('ComicFury/AmericanNerd'),
            cls('ComicFury/Amtheatre'),
            cls('ComicFury/Angstcomic'),
            cls('ComicFury/Applepine'),
            cls('ComicFury/Area42', 'mis'),
            cls('ComicFury/Atm'),
            cls('ComicFury/Atomicmonkey'),
            cls('ComicFury/Baseballcapsandtiaras'),
            cls('ComicFury/BATB'),
            cls('ComicFury/BetweenRounds'),
            cls('ComicFury/BigBookOfLameJokes'),
            cls('ComicFury/BiMorphon'),
            cls('ComicFury/Blessings'),
            cls('ComicFury/BrokenReality'),
            cls('ComicFury/BTTF'),
            cls('ComicFury/Cannonadeofhogwash'),
            cls('ComicFury/CatHero'),
            cls('ComicFury/Chocolava'),
            cls('ComicFury/ChristianHumberReloaded'),
            cls('ComicFury/Cockeyed'),
            cls('ComicFury/CoftheA'),
            cls('ComicFury/CompanyMan'),
            cls('ComicFury/Complicatedd'),
            cls('ComicFury/Conplicated'),
            cls('ComicFury/Crowbar'),
            cls('ComicFury/Crowbars'),
            cls('ComicFury/Curvyboneyosis'),
            cls('ComicFury/Dandk'),
            cls('ComicFury/Davidandtherobot'),
            cls('ComicFury/DenizensAttentionComic'),
            cls('ComicFury/Disturbingcomics'),
            cls('ComicFury/Docapoc'),
            cls('ComicFury/DucksMisery'),
            cls('ComicFury/Elfcomic'),
            cls('ComicFury/EMT'),
            cls('ComicFury/EternityC'),
            cls('ComicFury/Fathead'),
            cls('ComicFury/Fexpression'),
            cls('ComicFury/FireBorn2'),
            cls('ComicFury/Foxtales'),
            cls('ComicFury/Fpk'),
            cls('ComicFury/Ghostassassin'),
            cls('ComicFury/Gillimurphy'),
            cls('ComicFury/Glomshire'),
            cls('ComicFury/GodGames', 'mov'),
            cls('ComicFury/Goldrushdynllewcomics'),
            cls('ComicFury/Grandline3point5'),
            cls('ComicFury/Halloween2012'),
            cls('ComicFury/Halloween2013'),
            cls('ComicFury/HIRI'),
            cls('ComicFury/Hitmen'),
            cls('ComicFury/Honeyvenom'),
            cls('ComicFury/Insanitycorp'),
            cls('ComicFury/Inviziblecomixgroup'),
            cls('ComicFury/Isb'),
            cls('ComicFury/Its'),
            cls('ComicFury/Jenfferscartoonphotomanipulaion'),
            cls('ComicFury/Jenffersshow'),
            cls('ComicFury/Jeremy'),
            cls('ComicFury/Joysworldcomic'),
            cls('ComicFury/Judgedred'),
            cls('ComicFury/Jump2'),
            cls('ComicFury/Kachingcomic'),
            cls('ComicFury/Kazaandgwenna'),
            cls('ComicFury/Kevinzombie'),
            cls('ComicFury/Kindergardencrisis'),
            cls('ComicFury/Kirahitogame'),
            cls('ComicFury/Ladyspectra'),
            cls('ComicFury/Lastcallcomic'),
            cls('ComicFury/Lazy'),
            cls('ComicFury/Lena'),
            cls('ComicFury/Letitride'),
            cls('ComicFury/Lola2'),
            cls('ComicFury/LORDDARKE'),
            cls('ComicFury/Lp'),
            cls('ComicFury/LucidsDream'),
            cls('ComicFury/Lvl30psy'),
            cls('ComicFury/Maddog'),
            cls('ComicFury/Magisa'),
            cls('ComicFury/Midnightpeanutbutter'),
            cls('ComicFury/Minarga'),
            cls('ComicFury/MoizmadComix'),
            cls('ComicFury/Moths'),
            cls('ComicFury/MuttInTheMiddle'),
            cls('ComicFury/MyHorribleSite'),
            cls('ComicFury/Neighborscomic'),
            cls('ComicFury/Nojetpack'),
            cls('ComicFury/NoSongs'),
            cls('ComicFury/Nostalgiaofeden'),
            cls('ComicFury/Ocarinaoftim'),
            cls('ComicFury/OffHours'),
            cls('ComicFury/OldSchoolRasputinCatamite'),
            cls('ComicFury/Pandemonium'),
            cls('ComicFury/Paperstreamer'),
            cls('ComicFury/Peepsnperks'),
            cls('ComicFury/PersonaFTW'),
            cls('ComicFury/Pilgrimsprogress'),
            cls('ComicFury/PiratesLife'),
            cls('ComicFury/PobrePucho'),
            cls('ComicFury/Poussiere'),
            cls('ComicFury/Pt'),
            cls('ComicFury/Punch'),
            cls('ComicFury/Rangerrandom'),
            cls('ComicFury/Raspcat'),
            cls('ComicFury/RealLifeTrips'),
            cls('ComicFury/ReiketsuouNoKimi'),
            cls('ComicFury/RIDDICKQLOSSTALES'),
            cls('ComicFury/Romanjack'),
            cls('ComicFury/RPS'),
            cls('ComicFury/RPT'),
            cls('ComicFury/Rvr'),
            cls('ComicFury/Sarakleeyo'),
            cls('ComicFury/Sawbladersblacknuzlocke'),
            cls('ComicFury/Schizmatic'),
            cls('ComicFury/Seconds'),
            cls('ComicFury/Serengetti'),
            cls('ComicFury/SHADOWQUEEN'),
            cls('ComicFury/Shonenpunkremix'),
            cls('ComicFury/Sinjetpack'),
            cls('ComicFury/Spf1337'),
            cls('ComicFury/Sscomic'),
            cls('ComicFury/SteamSword'),
            cls('ComicFury/Teenagedragon'),
            cls('ComicFury/Theashes'),
            cls('ComicFury/TheButterflyEffect'),
            cls('ComicFury/Thecrease'),
            cls('ComicFury/TheGuardiansOfGrey'),
            cls('ComicFury/Tiziana'),
            cls('ComicFury/TwentyQuidAmusements'),
            cls('ComicFury/Underscore'),
            cls('ComicFury/ValtersRebellion'),
            cls('ComicFury/Wowwithatwistdamaclesandkejallcomic'),
            cls('ComicFury/YouAreNowEnteringAshburg'),
            cls('ComicGenesis/CryHavoc'),
            cls('ComicGenesis/IBlameDanny'),
            cls('ComicGenesis/SueosdelSur'),
            cls('Commissioned'),
            cls('CoolCatStudio'),
            cls('CowboyJedi', 'brk'),
            cls('Creators/BCinSpanish'),
            cls('Creators/GirlsandSportsinSpanish'),
            cls('Creators/Recess'),
            cls('Creators/RugratsinSpanish'),
            cls('CtrlAltDel/Sillies'),
            cls('DailyDose'),
            cls('DamnLol'),
            cls('DeathToTheExtremist'),
            cls('DoctorCat', 'brk'),
            cls('EerieCuties'),
            cls('Ellerbisms'),
            cls('Eriadan'),
            cls('EverydayBlues'),
            cls('FeyWinds'),
            cls('FilibusterCartoons'),
            cls('FowlLanguage', 'block'),
            cls('GlassHalfEmpty'),
            cls('GoComics/ABootsAndPupComic'),
            cls('GoComics/AdventuresofDaisy'),
            cls('GoComics/AdventuresofMartyandTurkey'),
            cls('GoComics/AdventuresofMikeAndSimon'),
            cls('GoComics/AgentGates'),
            cls('GoComics/AlisonWard'),
            cls('GoComics/AmaZnEvents'),
            cls('GoComics/AnythingGoes'),
            cls('GoComics/BCEnEspaol'),
            cls('GoComics/BenAndSeymour'),
            cls('GoComics/BeneaththeFerns'),
            cls('GoComics/BenSargent'),
            cls('GoComics/BERSERKALERT'),
            cls('GoComics/BestInShow'),
            cls('GoComics/BiffAndRiley'),
            cls('GoComics/BillyAndCo'),
            cls('GoComics/BlackboardDaze'),
            cls('GoComics/BobtheGroanUP'),
            cls('GoComics/Boogerbrain'),
            cls('GoComics/BotBrothers'),
            cls('GoComics/BrilliantMines'),
            cls('GoComics/BuffaloChips'),
            cls('GoComics/BuzzaWuzza'),
            cls('GoComics/CafConLeche'),
            cls('GoComics/CalAndOzz'),
            cls('GoComics/CandyPills'),
            cls('GoComics/Cartertoons'),
            cls('GoComics/ChanLowe'),
            cls('GoComics/ChasingUnicorns'),
            cls('GoComics/ChubbyGirlComics'),
            cls('GoComics/Classifudds'),
            cls('GoComics/CleoandCompany'),
            cls('GoComics/CockroachComix'),
            cls('GoComics/CoffeeShopTidbits'),
            cls('GoComics/Cortoons'),
            cls('GoComics/CowSheepandaGnomeNamedHelga'),
            cls('GoComics/DabneyandDad'),
            cls('GoComics/DevinCraneComicStripGhostwriter'),
            cls('GoComics/DialHforHBomb'),
            cls('GoComics/DitzAbledPrincess'),
            cls('GoComics/DoodleDaysComics'),
            cls('GoComics/Dromo'),
            cls('GoComics/EBEJeebie'),
            cls('GoComics/EDITORIALPASTANDPRESENT'),
            cls('GoComics/ElephantintheRoom'),
            cls('GoComics/EleriMaiHarrisCartoons'),
            cls('GoComics/ElfandMotorbelly'),
            cls('GoComics/ElMundoDeBeakman'),
            cls('GoComics/EngagAndNevets'),
            cls('GoComics/EspressoCity'),
            cls('GoComics/EttoreandBaldo'),
            cls('GoComics/FantasticMegaLeague'),
            cls('GoComics/FarcesofNature'),
            cls('GoComics/Featherweight'),
            cls('GoComics/FrankBlunt'),
            cls('GoComics/FrizziToons'),
            cls('GoComics/FundayMorning'),
            cls('GoComics/GatorsAndSuch'),
            cls('GoComics/GenerationMute'),
            cls('GoComics/GentleCreatures'),
            cls('GoComics/GetAGrip'),
            cls('GoComics/GlennMcCoy'),
            cls('GoComics/GoComicsontheRoad'),
            cls('GoComics/HamShears'),
            cls('GoComics/HanginOut'),
            cls('GoComics/HankandDalesOurWorld'),
            cls('GoComics/HanktheSock'),
            cls('GoComics/HarambeeHills'),
            cls('GoComics/Hbenson7'),
            cls('GoComics/HeadComics'),
            cls('GoComics/HIP'),
            cls('GoComics/HolidayDoodles'),
            cls('GoComics/HolySchnark'),
            cls('GoComics/HoodootheUnwiseOwl'),
            cls('GoComics/Humoresque'),
            cls('GoComics/ImaDillo'),
            cls('GoComics/ImTellingMom'),
            cls('GoComics/InheritTheMirth'),
            cls('GoComics/JackRadioComics'),
            cls('GoComics/JayAndBoneheadTheMunkysMrCowhide'),
            cls('GoComics/JustPosted'),
            cls('GoComics/KatetheGreat'),
            cls('GoComics/KirbysTreehouse'),
            cls('GoComics/KozmooftheCosmos'),
            cls('GoComics/LardWantsWorldPeace'),
            cls('GoComics/LarryvilleBlue'),
            cls('GoComics/Leadbellies'),
            cls('GoComics/LegendofBill'),
            cls('GoComics/LeGooseyLu'),
            cls('GoComics/LeighLunaComics'),
            cls('GoComics/LIGHTERSIDE'),
            cls('GoComics/LostInTranslation'),
            cls('GoComics/Lucan'),
            cls('GoComics/LucasLuminous'),
            cls('GoComics/Mac'),
            cls('GoComics/Markonpaper'),
            cls('GoComics/MaryBWary'),
            cls('GoComics/MassiveFalls'),
            cls('GoComics/McArroni'),
            cls('GoComics/Mick'),
            cls('GoComics/MidLifewAlan'),
            cls('GoComics/Millennialhood'),
            cls('GoComics/MinimumSecurity'),
            cls('GoComics/MixedMedications'),
            cls('GoComics/Molebashed'),
            cls('GoComics/Mortimer'),
            cls('GoComics/MrGigiAndTheSquid'),
            cls('GoComics/MrMorris'),
            cls('GoComics/Mulligan'),
            cls('GoComics/MyGuardianGrandpa'),
            cls('GoComics/NavyBean'),
            cls('GoComics/NeatStep'),
            cls('GoComics/NedAndLarry'),
            cls('GoComics/NeighborhoodZone'),
            cls('GoComics/NobodysHome'),
            cls('GoComics/NoPlaceLikeHolmes'),
            cls('GoComics/Norman'),
            cls('GoComics/Oat'),
            cls('GoComics/ObamaandtheFatman'),
            cls('GoComics/OntheQuad'),
            cls('GoComics/OrangesareFunny'),
            cls('GoComics/Outnumbered'),
            cls('GoComics/ParisDoodles'),
            cls('GoComics/Peanizles'),
            cls('GoComics/PetFood'),
            cls('GoComics/Pi'),
            cls('GoComics/PigtimesCartoon'),
            cls('GoComics/PipethePelican'),
            cls('GoComics/PlanB'),
            cls('GoComics/PlasticBabyHeadsfromOuterSpace'),
            cls('GoComics/PlentyofPenguins'),
            cls('GoComics/Poptropica'),
            cls('GoComics/Putz'),
            cls('GoComics/QuestionsForKids'),
            cls('GoComics/RandomActsofNancy'),
            cls('GoComics/RicigsToonTrivia'),
            cls('GoComics/RogueSymmetry'),
            cls('GoComics/Sabine'),
            cls('GoComics/SantavsDracula'),
            cls('GoComics/SCAIRYTALESTheNotSoScaryFairyTales'),
            cls('GoComics/Scurvyville'),
            cls('GoComics/SecondPrize'),
            cls('GoComics/Skooled'),
            cls('GoComics/SNAFU'),
            cls('GoComics/SouptoNutz'),
            cls('GoComics/SpaceNutz'),
            cls('GoComics/SPACESLUGS'),
            cls('GoComics/SpaceTimeFunnies'),
            cls('GoComics/Starslip'),
            cls('GoComics/STEPDAD'),
            cls('GoComics/Stookie'),
            cls('GoComics/SuburbanWilderness'),
            cls('GoComics/SuckerHeadSmack'),
            cls('GoComics/TeacherInk'),
            cls('GoComics/ThatMonkeyTune'),
            cls('GoComics/TheAcerbicCaf'),
            cls('GoComics/TheAdventuresofTeetyBallerina'),
            cls('GoComics/TheEdperiment'),
            cls('GoComics/TheFruitBowl'),
            cls('GoComics/TheGoldenKid'),
            cls('GoComics/TheInsolentLemon'),
            cls('GoComics/TheLightedLab'),
            cls('GoComics/TheLilMiesters'),
            cls('GoComics/TheOdderLimits'),
            cls('GoComics/THESILVERLINING'),
            cls('GoComics/TheSingleDadDiaries'),
            cls('GoComics/TheVernalPool'),
            cls('GoComics/TheWorstThingIveEverDone'),
            cls('GoComics/ThrompTM'),
            cls('GoComics/ToBeNamed'),
            cls('GoComics/TodaysDogg'),
            cls('GoComics/TonyAuth'),
            cls('GoComics/Toocrazy'),
            cls('GoComics/TOWHOMITMAYCONCERN'),
            cls('GoComics/Twaggies'),
            cls('GoComics/TwoBits'),
            cls('GoComics/Vernscartoons'),
            cls('GoComics/WayOutInLeftField'),
            cls('GoComics/WendlesLife'),
            cls('GoComics/Whatcatscanandcantdo'),
            cls('GoComics/WickedCrispy'),
            cls('GoComics/WindingRoads'),
            cls('GoComics/WitoftheWorld'),
            cls('GoComics/YouCanWithBeakmanAndJax'),
            cls('GoComics/YouGuysAreMyFriendsTheComic'),
            cls('GoComics/ZacharyNixonJohnson'),
            cls('GunnerkrigCourt'),
            cls('HorribleVille'),
            cls('JerkCity'),
            cls('KatzenfutterGeleespritzer'),
            cls('KeenSpot/Adventurers', 'mov'),
            cls('KeenSpot/Landis'),
            cls('Key'),
            cls('KillerKomics'),
            cls('Kukuburi'),
            cls('Lint'),
            cls('LinuxComFridayFunnies'),
            cls('NekkoAndJoruba'),
            cls('NekoTheKitty'),
            cls('NewAdventuresOfBobbin'),
            cls('Nnewts'),
            cls('OddFish'),
            cls('OneQuestion'),
            cls('OrnerBoy'),
            cls('PensAndTales/Evilish'),
            cls('PensAndTales/FireflyCross'),
            cls('PetiteSymphony/Kickinrad'),
            cls('PetiteSymphony/Orangegrind'),
            cls('PetiteSymphony/Seed'),
            cls('Pimpette'),
            cls('PunksAndNerds', 'mis'),
            cls('PunksAndNerdsOld'),
            cls('RedsPlanet'),
            cls('RedString'),
            cls('SmackJeeves/Aarrevaara'),
            cls('SmackJeeves/AchievementStuck'),
            cls('SmackJeeves/AGirlAndHerShadow'),
            cls('SmackJeeves/Allthatglitters'),
            cls('SmackJeeves/AloversRule'),
            cls('SmackJeeves/Anathemacomics'),
            cls('SmackJeeves/AngelBeast'),
            cls('SmackJeeves/ArchportCityChronicles'),
            # moved to www.mgcomics.com, which has a robots.txt for everything
            cls('SmackJeeves/Aware', 'block'),
            cls('SmackJeeves/AwesomeSauce'),
            cls('SmackJeeves/Babywhatsyoursign'),
            cls('SmackJeeves/BetweenLightandDark'),
            cls('SmackJeeves/BetweenWorlds'),
            cls('SmackJeeves/BeyondTemptation'),
            cls('SmackJeeves/BLDShortComics'),
            cls('SmackJeeves/Bloodyfairytale'),
            cls('SmackJeeves/BLOT'),
            cls('SmackJeeves/BlueWell'),
            cls('SmackJeeves/BreakfastonaCliff'),
            cls('SmackJeeves/CafeAmargo'),
            cls('SmackJeeves/Captor'),
            cls('SmackJeeves/ChaosTheory2005'),
            cls('SmackJeeves/CleanCure'),
            cls('SmackJeeves/DaddysGirl'),
            cls('SmackJeeves/DeadtoDay'),
            cls('SmackJeeves/Debtsettlement'),
            cls('SmackJeeves/DebtSettlement2OperationExtinction'),
            cls('SmackJeeves/DefyingGravityTheFourGreatGuardians'),
            cls('SmackJeeves/Destinationunknown'),
            cls('SmackJeeves/DevilTrainee'),
            cls('SmackJeeves/DevilTraineeSpanish'),
            cls('SmackJeeves/Diexemor'),
            cls('SmackJeeves/DigisRandomSpriteshack'),
            cls('SmackJeeves/DontDie'),
            cls('SmackJeeves/ElfenLiedDifferences'),
            cls('SmackJeeves/Entuthrie'),
            cls('SmackJeeves/EozinKadonnutKuningas'),
            cls('SmackJeeves/EpicChaos'),
            cls('SmackJeeves/EternalKnights'),
            cls('SmackJeeves/EvD'),
            cls('SmackJeeves/FatetheAnthologyofKaienandhisfuckingmagicfriends'),
            cls('SmackJeeves/FeathersPI'),
            cls('SmackJeeves/FemmeSchism'),
            cls('SmackJeeves/FireWire'),
            cls('SmackJeeves/FrenzyRedux'),
            cls('SmackJeeves/FrogKing'),
            cls('SmackJeeves/FuckMyLife'),
            cls('SmackJeeves/FurtherDowntheRabbitHole'),
            cls('SmackJeeves/GATEKEEPER'),
            cls('SmackJeeves/GearTheTakedown'),
            cls('SmackJeeves/GoldenSunGenerationsAftermathVolume1'),
            cls('SmackJeeves/GoldenSunGenerationsColossoVolume6'),
            cls('SmackJeeves/GraveImpressions'),
            cls('SmackJeeves/GreenKirbyandabunchofotherpeopledoinstuff'),
            cls('SmackJeeves/GuardianGhost'),
            cls('SmackJeeves/Harfang'),
            cls('SmackJeeves/HIPS'),
            cls('SmackJeeves/HitandMiss'),
            cls('SmackJeeves/HotChocolate'),
            cls('SmackJeeves/Hybristorific'),
            cls('SmackJeeves/Ianua'),
            cls('SmackJeeves/ImminentMoose'),
            cls('SmackJeeves/InthePride'),
            cls('SmackJeeves/Intoxicated'),
            cls('SmackJeeves/Jantarpol'),
            cls('SmackJeeves/Knife'),
            cls('SmackJeeves/Kranburn'),
            cls('SmackJeeves/KuroNeko'),
            cls('SmackJeeves/LastLivingSouls'),
            cls('SmackJeeves/LegendsofMobiusBookOne'),
            cls('SmackJeeves/LetsBreakitforReals'),
            cls('SmackJeeves/LiliBleu'),
            cls('SmackJeeves/LoveTwister'),
            cls('SmackJeeves/MagicalGirlAlice'),
            cls('SmackJeeves/MasqueradeWTTM'),
            cls('SmackJeeves/MegaManBattleNetwork7'),
            cls('SmackJeeves/MegaManiacs'),
            cls('SmackJeeves/MerirosvotSeikkailumerella'),
            cls('SmackJeeves/MewsDynasty'),
            cls('SmackJeeves/MixupofallMixups'),
            cls('SmackJeeves/MomthegamestorerippedusoffAGAIN'),
            cls('SmackJeeves/MoonlitDawnAMythicalTale'),
            cls('SmackJeeves/MUTE', 'mov'),
            cls('SmackJeeves/MyBoyfriendisaMobBoss'),
            cls('SmackJeeves/MyTrollLife'),
            cls('SmackJeeves/MyTwoCentsPlusTax'),
            cls('SmackJeeves/NeoCrystalAdventures'),
            cls('SmackJeeves/NihilWandasJourney'),
            cls('SmackJeeves/OddContact'),
            cls('SmackJeeves/Ohman', 'brk'),
            cls('SmackJeeves/OneFrameGags'),
            cls('SmackJeeves/Pahantekija'),
            cls('SmackJeeves/Paradox'),
            cls('SmackJeeves/Paripety'),
            cls('SmackJeeves/Perinto'),
            cls('SmackJeeves/PerplexingMagnoliaDisruption', 'mov'),
            cls('SmackJeeves/PlatonicBoyfriends'),
            cls('SmackJeeves/Plotlessnesses'),
            cls('SmackJeeves/PokemonGleamingCrystal', 'mis'),
            cls('SmackJeeves/PokemonMysteryDungeonTeamCrystal'),
            cls('SmackJeeves/PRAGUERACE'),
            cls('SmackJeeves/PumpkinFlower'),
            cls('SmackJeeves/Razor'),
            cls('SmackJeeves/RedVelvetRequiem'),
            cls('SmackJeeves/RuderiQuest'),
            cls('SmackJeeves/SAKANA'),
            cls('SmackJeeves/SenoireDelirium'),
            cls('SmackJeeves/SerendipityAnEquestrianTale'),
            cls('SmackJeeves/ShacklesInstallment02'),
            cls('SmackJeeves/SimonSues'),
            cls('SmackJeeves/SimplySarah'),
            cls('SmackJeeves/SomebodyShootMe'),
            cls('SmackJeeves/SonicUniverseAsk'),
            cls('SmackJeeves/SoulGuardian'),
            cls('SmackJeeves/SpaghettiAndMeatballs'),
            cls('SmackJeeves/SparkStory', 'mis'),
            cls('SmackJeeves/Spidersilk', 'mov'),
            cls('SmackJeeves/Symbios'),
            cls('SmackJeeves/TechnicolorLondon'),
            cls('SmackJeeves/TeKscloset'),
            cls('SmackJeeves/TheAttackoftheRecoloursSeason1'),
            cls('SmackJeeves/TheAvianStories'),
            cls('SmackJeeves/TheCurtandTonyShow'),
            cls('SmackJeeves/TheDarkAgeofMobius'),
            cls('SmackJeeves/TheHobbitbic'),
            cls('SmackJeeves/ThehumanBEing'),
            cls('SmackJeeves/TheKeyToReality'),
            cls('SmackJeeves/TheLostland'),
            cls('SmackJeeves/TheMewExperiment'),
            cls('SmackJeeves/TheMoistTouch'),
            cls('SmackJeeves/TheRandomObscureFairyTaleNoOnesEverReallyHeardOf'),
            cls('SmackJeeves/TheSomewhereOther'),
            cls('SmackJeeves/TheWastelands', 'mis'),
            cls('SmackJeeves/ThinkBeforeYouThink', 'mov'),
            cls('SmackJeeves/ThroughTheWonkyEye'),
            cls('SmackJeeves/TitleUnrelated'),
            cls('SmackJeeves/TotalPokemonIsland'),
            cls('SmackJeeves/TrillyAndSilly'),
            cls('SmackJeeves/TRIPP'),
            cls('SmackJeeves/VampireFetish'),
            cls('SmackJeeves/WolfWolf'),
            cls('SmackJeeves/WonderTheatre'),
            cls('SmackJeeves/YouAreTheReasonForTheEndOfTheWorld'),
            cls('SnowFlakes'),
            cls('StrawberryDeathCake'),
            cls('Stubble'),
            cls('SuburbanTribe'),
            cls('TheOuterQuarter'),
            cls('TheParkingLotIsFull'),
            cls('TheThinHLine', 'acc'),
            cls('ThunderAndLightning'),
            cls('TinyKittenTeeth'),
            cls('TwoTwoOneFour'),
            cls('VampireCheerleaders'),
            cls('WayfarersMoon'),
            cls('WebcomicEu/Talandor'),
            cls('WebcomicEu/TheBessEffect'),
            cls('WebcomicEu/TheBessEffectEnglish'),
            cls('WebcomicsNation/AgnesQuill'),
            cls('WebcomicsNation/MyMuse'),
            cls('WebcomicsNation/NekkoAndJoruba'),
            cls('WeCanSleepTomorrow'),
            cls('WhiteNinja'),
            cls('WLP/ShadowChasers'),
            cls('WotNow'),

            # Removed in 3.0
            cls('ComicFury/AdventuresOftheGreatCaptainMaggieandCrew'),
            cls('ComicFury/AWAKENING'),
            cls('ComicFury/Beebleville'),
            cls('ComicFury/BloodLegaciesEternity'),
            cls('ComicFury/CharlesAndViktor'),
            cls('ComicFury/DemonWings'),
            cls('ComicFury/DnDDumbAndDumber'),
            cls('ComicFury/DoodlelandComics'),
            cls('ComicFury/Elements'),
            cls('ComicFury/FairyDust'),
            cls('ComicFury/FandomMisadventures'),
            cls('ComicFury/GreenerGrass'),
            cls('ComicFury/HelloWanderingStar'),
            cls('ComicFury/HINATATheDemonSlayer'),
            cls('ComicFury/Hodgemosh'),
            cls('ComicFury/Imp'),
            cls('ComicFury/Kitsune'),
            cls('ComicFury/LaszloAndEdgar'),
            cls('ComicFury/Maluk'),
            cls('ComicFury/MegamanComic'),
            cls('ComicFury/Mischeif'),
            cls('ComicFury/OceanLabyrinth'),
            cls('ComicFury/PatchworkPeople'),
            cls('ComicFury/PlanetChaser'),
            cls('ComicFury/PornographyInFiveActs'),
            cls('ComicFury/PoussireDeFe'),
            cls('ComicFury/RadioMustard'),
            cls('ComicFury/RaytoonsKids'),
            cls('ComicFury/RED'),
            cls('ComicFury/ResNullius'),
            cls('ComicFury/ResNulliusCS'),
            cls('ComicFury/Seed'),
            cls('ComicFury/SixteenCandlesHuntersAgency'),
            cls('ComicFury/TheAcryden'),
            cls('ComicFury/TheHourlyComic'),
            cls('ComicFury/TheKAMics'),
            cls('ComicFury/TheUnthinkableHybrid'),
            cls('ComicFury/TwentyFourSeven'),
            cls('ComicFury/TwentyFourSevenFans'),
            cls('ComicSherpa/060'),
            cls('ComicSherpa/AaronGuile'),
            cls('ComicSherpa/ABCStreet'),
            cls('ComicSherpa/ABitSketch'),
            cls('ComicSherpa/ABomb'),
            cls('ComicSherpa/ACMEINKD'),
            cls('ComicSherpa/AcornPark'),
            cls('ComicSherpa/Adulting'),
            cls('ComicSherpa/AllInGoodTime'),
            cls('ComicSherpa/AmandaTheGreat'),
            cls('ComicSherpa/AndNow'),
            cls('ComicSherpa/Anecdote'),
            cls('ComicSherpa/AnimalMitchell'),
            cls('ComicSherpa/AnneAndPythagoras'),
            cls('ComicSherpa/AppleCreekComics'),
            cls('ComicSherpa/ATasteOfTimes'),
            cls('ComicSherpa/BatchRejection'),
            cls('ComicSherpa/Bazoobee'),
            cls('ComicSherpa/BeMisery'),
            cls('ComicSherpa/BigJim'),
            cls('ComicSherpa/Bluebonnets'),
            cls('ComicSherpa/BlueSkiesToons'),
            cls('ComicSherpa/BobsYourUncle'),
            cls('ComicSherpa/BoltsAndNuts'),
            cls('ComicSherpa/Bork'),
            cls('ComicSherpa/BottAuto'),
            cls('ComicSherpa/BUNS'),
            cls('ComicSherpa/Bushscrubs'),
            cls('ComicSherpa/CAFFEINATED'),
            cls('ComicSherpa/CandacenCompany'),
            cls('ComicSherpa/CarteBlanche'),
            cls('ComicSherpa/CharmysArmy'),
            cls('ComicSherpa/Complex'),
            cls('ComicSherpa/CourageousManAdventures'),
            cls('ComicSherpa/DadsDay'),
            cls('ComicSherpa/DBCartoons'),
            cls('ComicSherpa/DoghouseInYourSoul'),
            cls('ComicSherpa/DoingTime'),
            cls('ComicSherpa/DontPickTheFlowers'),
            cls('ComicSherpa/Dragin'),
            cls('ComicSherpa/DumbQuestionBadAnswer'),
            cls('ComicSherpa/DungeonHordes'),
            cls('ComicSherpa/DustSpecks'),
            cls('ComicSherpa/DutchnPals'),
            cls('ComicSherpa/Econogirl'),
            cls('ComicSherpa/EightballEyeball'),
            cls('ComicSherpa/Elmo'),
            cls('ComicSherpa/Endangered'),
            cls('ComicSherpa/FamousAndNotSoFamousQuotes'),
            cls('ComicSherpa/FarOut'),
            cls('ComicSherpa/FatherOfTheBrood'),
            cls('ComicSherpa/FoxTheCat'),
            cls('ComicSherpa/FrankAndSteinway'),
            cls('ComicSherpa/FriedCritter'),
            cls('ComicSherpa/GarciaCartoonCo'),
            cls('ComicSherpa/GIRTH'),
            cls('ComicSherpa/GrandmaSnoops'),
            cls('ComicSherpa/GrannyAnny'),
            cls('ComicSherpa/GreenPieces'),
            cls('ComicSherpa/GunstonStreet'),
            cls('ComicSherpa/HallEditorialCartoons'),
            cls('ComicSherpa/HaphazardHumor'),
            cls('ComicSherpa/Headcheese'),
            cls('ComicSherpa/Hogwashed'),
            cls('ComicSherpa/HomeLife'),
            cls('ComicSherpa/Hubbel'),
            cls('ComicSherpa/HugoComics'),
            cls('ComicSherpa/HurrieTheMisManager'),
            cls('ComicSherpa/HuskyTales'),
            cls('ComicSherpa/InkwellForest'),
            cls('ComicSherpa/IronyOr'),
            cls('ComicSherpa/ItsJustJim'),
            cls('ComicSherpa/JolleyStuffBrowser'),
            cls('ComicSherpa/KALEECHIKORNERS'),
            cls('ComicSherpa/KartoonsByKline'),
            cls('ComicSherpa/LaffToons'),
            cls('ComicSherpa/LiliAndDerek'),
            cls('ComicSherpa/LilleysSillies'),
            cls('ComicSherpa/LimboRoad'),
            cls('ComicSherpa/LumAndAbner'),
            cls('ComicSherpa/MarysNature'),
            cls('ComicSherpa/Millennialville'),
            cls('ComicSherpa/Milton50'),
            cls('ComicSherpa/Mindframe'),
            cls('ComicSherpa/Minihahas'),
            cls('ComicSherpa/MiscSoup'),
            cls('ComicSherpa/MisterAndMe'),
            cls('ComicSherpa/MockAll'),
            cls('ComicSherpa/Moments'),
            cls('ComicSherpa/Mongrels'),
            cls('ComicSherpa/MortsIsland'),
            cls('ComicSherpa/MySonIsADog'),
            cls('ComicSherpa/NoAmbiguity'),
            cls('ComicSherpa/NoBusinessIKnow'),
            cls('ComicSherpa/NoOrdinaryLife'),
            cls('ComicSherpa/Npchumorcom'),
            cls('ComicSherpa/OneFunnyGoldenRetriever'),
            cls('ComicSherpa/ONIONAndPEA'),
            cls('ComicSherpa/OscarAndAnnie'),
            cls('ComicSherpa/OverQuirked'),
            cls('ComicSherpa/PaddedCell'),
            cls('ComicSherpa/Painterly'),
            cls('ComicSherpa/PalAndBuddy'),
            cls('ComicSherpa/PawsForThoughtComics'),
            cls('ComicSherpa/Peeples'),
            cls('ComicSherpa/PeopleOfEarth'),
            cls('ComicSherpa/PicpakDog'),
            cls('ComicSherpa/PirateMike'),
            cls('ComicSherpa/PoliceLimit'),
            cls('ComicSherpa/PoliticularJokesAndRuffus'),
            cls('ComicSherpa/Prideland'),
            cls('ComicSherpa/PrimusTheBadPhilosopher'),
            cls('ComicSherpa/ProfessorHerbertAndGEO'),
            cls('ComicSherpa/QueenBlackbeard'),
            cls('ComicSherpa/QuickDraw'),
            cls('ComicSherpa/RandysRationale'),
            cls('ComicSherpa/Ringers'),
            cls('ComicSherpa/RonWarren'),
            cls('ComicSherpa/SandSharkBeach'),
            cls('ComicSherpa/SharpCurveComics'),
            cls('ComicSherpa/SherpaAid'),
            cls('ComicSherpa/SignGarden'),
            cls('ComicSherpa/SignsOfAFrustratedGolfer'),
            cls('ComicSherpa/Skull'),
            cls('ComicSherpa/Skylarking'),
            cls('ComicSherpa/SleepytownBeagles'),
            cls('ComicSherpa/SmallNerdyCreatures'),
            cls('ComicSherpa/Smith'),
            cls('ComicSherpa/Snootle'),
            cls('ComicSherpa/SoccerDude'),
            cls('ComicSherpa/SoccerEarth'),
            cls('ComicSherpa/SOD'),
            cls('ComicSherpa/SomethingAboutCeleste'),
            cls('ComicSherpa/SookyRottweiler'),
            cls('ComicSherpa/Spaceport51'),
            cls('ComicSherpa/SportsByVoort'),
            cls('ComicSherpa/StaleCrackers'),
            cls('ComicSherpa/StankoAndTibor'),
            cls('ComicSherpa/Strangeville'),
            cls('ComicSherpa/SubSub'),
            cls('ComicSherpa/SuburbanFairyTales'),
            cls('ComicSherpa/SUITSANDGUARDERS'),
            cls('ComicSherpa/SuperSiblings'),
            cls('ComicSherpa/TheBeauforts'),
            cls('ComicSherpa/TheBellies'),
            cls('ComicSherpa/TheBoobiehatch'),
            cls('ComicSherpa/TheCardinal'),
            cls('ComicSherpa/TheDinkledorfs'),
            cls('ComicSherpa/TheFabulousBushPigs'),
            cls('ComicSherpa/TheGrayZone'),
            cls('ComicSherpa/TheGreenMonkeys'),
            cls('ComicSherpa/TheMagicForest'),
            cls('ComicSherpa/TheMothManAndLarvaeBoy'),
            cls('ComicSherpa/TheMountainMen'),
            cls('ComicSherpa/TheNeighborhood'),
            cls('ComicSherpa/TheNevilleYouKnow'),
            cls('ComicSherpa/TheNonsenseNewz'),
            cls('ComicSherpa/TheOldManAndHisDog'),
            cls('ComicSherpa/TheQuinnAndFinnShow'),
            cls('ComicSherpa/TheRocks'),
            cls('ComicSherpa/TheUnemployed'),
            cls('ComicSherpa/Thingsesque'),
            cls('ComicSherpa/TodaysTrump'),
            cls('ComicSherpa/TopicToons'),
            cls('ComicSherpa/ToughTown'),
            cls('ComicSherpa/ToxicValues'),
            cls('ComicSherpa/TruthBeKnown'),
            cls('ComicSherpa/TuesdaysWithCory'),
            cls('ComicSherpa/Underdone'),
            cls('ComicSherpa/UnMannerlyWays'),
            cls('ComicSherpa/ViewFromTheCouch'),
            cls('ComicSherpa/VoicesInTheDark'),
            cls('ComicSherpa/WarpedAndDemented'),
            cls('ComicSherpa/Waskataskahiskewaskewan'),
            cls('ComicSherpa/WayOutComics'),
            cls('ComicSherpa/WeaselInk'),
            cls('ComicSherpa/WhiskeyFalls'),
            cls('ComicSherpa/Windsock'),
            cls('ComicSherpa/WrobbertCartoons'),
            cls('ComicSherpa/YinYangster'),
            cls('ComicSherpa/ZombieHeights'),
            cls('ComicSherpa/Zootopia'),
            cls('ComicsKingdom/BabyBlues'),
            cls('ComicsKingdom/BettyBoopSundays'),
            cls('ComicsKingdom/Buckles'),
            cls('ComicsKingdom/JohnnyHazardSundays'),
            cls('ComicsKingdom/Redeye'),
            cls('ComicsKingdom/RedeyeSundays'),
            cls('CrapIDrewOnMyLunchBreak'),
            cls('GoComics/060'),
            cls('GoComics/2CowsAndAChicken'),
            cls('GoComics/ABitSketch'),
            cls('GoComics/AmericanChopSuey'),
            cls('GoComics/Andnow'),
            cls('GoComics/Anecdote'),
            cls('GoComics/AppleCreekComics'),
            cls('GoComics/AskACat'),
            cls('GoComics/AskAPortlySyndicatePerson'),
            cls('GoComics/BabyTrump'),
            cls('GoComics/BadReporter'),
            cls('GoComics/BarkingCrayon'),
            cls('GoComics/Bazoobee'),
            cls('GoComics/Bewley'),
            cls('GoComics/Biographic'),
            cls('GoComics/Bluebonnets'),
            cls('GoComics/BlueSkiesToons'),
            cls('GoComics/BottAuto'),
            cls('GoComics/BrainSquirts'),
            cls('GoComics/BUNS'),
            cls('GoComics/BushyTales'),
            cls('GoComics/CAFFEINATED'),
            cls('GoComics/CapsulasMedicas'),
            cls('GoComics/CharmysArmy'),
            cls('GoComics/CheapThrillsCuisine'),
            cls('GoComics/ClearBlueWater'),
            cls('GoComics/Committed'),
            cls('GoComics/ConnieToTheWonnie'),
            cls('GoComics/CourageousManAdventures'),
            cls('GoComics/DanWasserman'),
            cls('GoComics/DontPicktheFlowers'),
            cls('GoComics/DorrisMcComics'),
            cls('GoComics/Dragin'),
            cls('GoComics/DragonGirl'),
            cls('GoComics/Drive'),
            cls('GoComics/DudeAndDude'),
            cls('GoComics/DumbQuestionBadAnswer'),
            cls('GoComics/DustSpecks'),
            cls('GoComics/Econogirl'),
            cls('GoComics/EdgeOfAdventure'),
            cls('GoComics/Elmo'),
            cls('GoComics/EricTheCircle'),
            cls('GoComics/FacesOfTheNewsByKerryWaghorn'),
            cls('GoComics/FarOut'),
            cls('GoComics/FortKnox'),
            cls('GoComics/FrankAndSteinway'),
            cls('GoComics/FriedCritter'),
            cls('GoComics/GarciaCartoonCo'),
            cls('GoComics/GarfieldMinusGarfield'),
            cls('GoComics/GIRTH'),
            cls('GoComics/GnomeSyndicate'),
            cls('GoComics/GoComicsFanArt'),
            cls('GoComics/Graffiti'),
            cls('GoComics/GrannyAnny'),
            cls('GoComics/GreenPieces'),
            cls('GoComics/GunstonStreet'),
            cls('GoComics/HaikuEwe'),
            cls('GoComics/Headcheese'),
            cls('GoComics/HealthCapsules'),
            cls('GoComics/HowToCat'),
            cls('GoComics/HUBRIS'),
            cls('GoComics/HumanCull'),
            cls('GoComics/InspectorDangersCrimeQuiz'),
            cls('GoComics/ItsjustJim'),
            cls('GoComics/JerryHolbert'),
            cls('GoComics/JillpokeBohemia'),
            cls('GoComics/JimsJournal'),
            cls('GoComics/JoeVanilla'),
            cls('GoComics/JoeyAlisonSayersComics'),
            cls('GoComics/JustSayUncle'),
            cls('GoComics/KartoonsByKline'),
            cls('GoComics/KenCatalino'),
            cls('GoComics/KidSpot'),
            cls('GoComics/KidTown'),
            cls('GoComics/KitNCarlyle'),
            cls('GoComics/LostSideOfSuburbia'),
            cls('GoComics/LumandAbner'),
            cls('GoComics/MagicInAMinute'),
            cls('GoComics/Magnificatz'),
            cls('GoComics/MazeToonsPuzzle'),
            cls('GoComics/MegClassics'),
            cls('GoComics/MichaelAndrew'),
            cls('GoComics/Microcosm'),
            cls('GoComics/Millennialville'),
            cls('GoComics/Milton50'),
            cls('GoComics/Mindframe'),
            cls('GoComics/MiscSoup'),
            cls('GoComics/MisterAndMe'),
            cls('GoComics/Mo'),
            cls('GoComics/MomsCancer'),
            cls('GoComics/MortsIsland'),
            cls('GoComics/MustardAndBoloney'),
            cls('GoComics/MyCage'),
            cls('GoComics/MyCageNewAndOld'),
            cls('GoComics/NoOrdinaryLife'),
            cls('GoComics/ONIONAndPEA'),
            cls('GoComics/PaddedCell'),
            cls('GoComics/PaulSzep'),
            cls('GoComics/Peeples'),
            cls('GoComics/PicturesInBoxes'),
            cls('GoComics/PieComic'),
            cls('GoComics/Pinkerton'),
            cls('GoComics/PirateMike'),
            cls('GoComics/PoliceLimit'),
            cls('GoComics/PopCultureShockTherapy'),
            cls('GoComics/PromisesPromises'),
            cls('GoComics/ReplyAll'),
            cls('GoComics/ReplyAllLite'),
            cls('GoComics/RonWarren'),
            cls('GoComics/Sheldon'),
            cls('GoComics/Shoecabbage'),
            cls('GoComics/Shortcuts'),
            cls('GoComics/ShutterbugFollies'),
            cls('GoComics/SignGarden'),
            cls('GoComics/SleepytownBeagles'),
            cls('GoComics/SmallNerdyCreatures'),
            cls('GoComics/Smith'),
            cls('GoComics/SoccerEarth'),
            cls('GoComics/SookyRottweiler'),
            cls('GoComics/Speechless'),
            cls('GoComics/SportsbyVoort'),
            cls('GoComics/StankoAndTibor'),
            cls('GoComics/Starling'),
            cls('GoComics/SubSub'),
            cls('GoComics/SuburbanFairyTales'),
            cls('GoComics/SuperSiblings'),
            cls('GoComics/ThatsLife'),
            cls('GoComics/TheBeauforts'),
            cls('GoComics/TheBentPinky'),
            cls('GoComics/TheBestMedicineCartoon'),
            cls('GoComics/TheBoobiehatch'),
            cls('GoComics/TheCardinal'),
            cls('GoComics/TheConjurers'),
            cls('GoComics/TheCreeps'),
            cls('GoComics/TheGentlemansArmchair'),
            cls('GoComics/TheGreenMonkeys'),
            cls('GoComics/TheLastMechanicalMonster'),
            cls('GoComics/TheLeftyBoscoPictureShow'),
            cls('GoComics/TheLostBear'),
            cls('GoComics/TheNorm40'),
            cls('GoComics/TheOldManAndHisDog'),
            cls('GoComics/TheOtherEnd'),
            cls('GoComics/TheQuinnAndFinnShow'),
            cls('GoComics/TheQuixoteSyndrome'),
            cls('GoComics/TheSunshineClub'),
            cls('GoComics/Thingsesque'),
            cls('GoComics/TimEagan'),
            cls('GoComics/TOBY'),
            cls('GoComics/ToughTown'),
            cls('GoComics/Trivquiz'),
            cls('GoComics/UncleArtsFunland'),
            cls('GoComics/USAcres'),
            cls('GoComics/WayOutComics'),
            cls('GoComics/WhiskeyFalls'),
            cls('GoComics/WhyattCartoons'),
            cls('GoComics/Winston'),
            cls('GoComics/WorldOfWonder'),
            cls('GoComics/Wrobbertcartoons'),
            cls('GoComics/Zootopia'),
            cls('JustAnotherEscape'),
            cls('Laiyu', 'brk'),
            cls('MangaDex/DrStone', 'legal'),
            cls('MangaDex/HeavensDesignTeam', 'legal'),
            cls('MangaDex/SPYxFAMILY', 'legal'),
            cls('Ryugou'),
            cls('SeelPeel'),
            cls('Shivae/CafeAnime'),
            cls('Shivae/Extras'),
            cls('SmackJeeves/20TimesKirby'),
            cls('SmackJeeves/2Kingdoms'),
            cls('SmackJeeves/355Days'),
            cls('SmackJeeves/_A_'),
            cls('SmackJeeves/AB'),
            cls('SmackJeeves/AceOfHearts'),
            cls('SmackJeeves/AcidMonday'),
            cls('SmackJeeves/Adalsysla'),
            cls('SmackJeeves/ADoodleADay'),
            cls('SmackJeeves/AdventuresOfLumAndFriends'),
            cls('SmackJeeves/AdventuresoftheWeird'),
            cls('SmackJeeves/AetherTheories'),
            cls('SmackJeeves/AgeOfTheGray'),
            cls('SmackJeeves/AGirlOnTheServer'),
            cls('SmackJeeves/AKirbyKomic'),
            cls('SmackJeeves/ALaMode'),
            cls('SmackJeeves/AllInLOVE'),
            cls('SmackJeeves/AllStarHeroes'),
            cls('SmackJeeves/AlmostTouching'),
            cls('SmackJeeves/AlwaysDamnedWebcomic'),
            cls('SmackJeeves/AlwaysRainingHere'),
            cls('SmackJeeves/Amaravati'),
            cls('SmackJeeves/AmorVincitOmnia'),
            cls('SmackJeeves/AmsdenEstate'),
            cls('SmackJeeves/AngelGuardian'),
            cls('SmackJeeves/ANGELOU'),
            cls('SmackJeeves/AnimalAdventures'),
            cls('SmackJeeves/Animayhem'),
            cls('SmackJeeves/AnythingAboutNothing'),
            cls('SmackJeeves/APTComic'),
            cls('SmackJeeves/AQuestionOfCharacter'),
            cls('SmackJeeves/Area9'),
            cls('SmackJeeves/AroundTheBlock'),
            cls('SmackJeeves/ArtOfAFantasy'),
            cls('SmackJeeves/ASongforElise'),
            cls('SmackJeeves/AtArmsLength'),
            cls('SmackJeeves/Atlaswebcomic'),
            cls('SmackJeeves/Autophobia'),
            cls('SmackJeeves/AyaTakeo'),
            cls('SmackJeeves/AYuriCollab'),
            cls('SmackJeeves/BabysittingFourDemons'),
            cls('SmackJeeves/BadassRiz'),
            cls('SmackJeeves/BallandChain'),
            cls('SmackJeeves/Bard'),
            cls('SmackJeeves/BassComicAdventures'),
            cls('SmackJeeves/BattleSequence'),
            cls('SmackJeeves/Bearhoney'),
            cls('SmackJeeves/BearlyAbel'),
            cls('SmackJeeves/BeautifulLies'),
            cls('SmackJeeves/BehindTheGlassCurtain'),
            cls('SmackJeeves/BehindTheObsidianMirror'),
            cls('SmackJeeves/BeretCatComics'),
            cls('SmackJeeves/Bestbrosforever'),
            cls('SmackJeeves/Betovering'),
            cls('SmackJeeves/BettencourtHotel'),
            cls('SmackJeeves/BeTwin'),
            cls('SmackJeeves/BeyondTheOrdinary'),
            cls('SmackJeeves/BioRevelation'),
            cls('SmackJeeves/Bl3'),
            cls('SmackJeeves/BlackAndBlue'),
            cls('SmackJeeves/Blackdemon'),
            cls('SmackJeeves/BlackDragon'),
            cls('SmackJeeves/BlackFridayRule'),
            cls('SmackJeeves/BlackSheepcomic'),
            cls('SmackJeeves/BleachRedux'),
            cls('SmackJeeves/BlindandBlue'),
            cls('SmackJeeves/BloodhuntersBirthOfAVampire'),
            cls('SmackJeeves/BloomAPokemonConquestComic'),
            cls('SmackJeeves/BlueHair'),
            cls('SmackJeeves/BoilingPointofBrain'),
            cls('SmackJeeves/BoogeyDancingMonkeyPot'),
            cls('SmackJeeves/BreachOfAgency'),
            cls('SmackJeeves/Burn'),
            cls('SmackJeeves/CafeSuada'),
            cls('SmackJeeves/Cambion'),
            cls('SmackJeeves/CaptiveSoul'),
            cls('SmackJeeves/Captured'),
            cls('SmackJeeves/CaravanaTaleofGodsandMen'),
            cls('SmackJeeves/Cataclysm'),
            cls('SmackJeeves/Catnip'),
            cls('SmackJeeves/Cerintha'),
            cls('SmackJeeves/ChampionofChampions'),
            cls('SmackJeeves/ChampionsAndHeroesAgeOfDragons'),
            cls('SmackJeeves/ChannelDDDNews'),
            cls('SmackJeeves/ChaosAdventuresII'),
            cls('SmackJeeves/ChaoticNation'),
            cls('SmackJeeves/Charaktermaske'),
            cls('SmackJeeves/Chatuplines'),
            cls('SmackJeeves/CheneysGotaGun'),
            cls('SmackJeeves/ChickenScratches'),
            cls('SmackJeeves/ChildrenOfTheNight'),
            cls('SmackJeeves/ChimiMouryou'),
            cls('SmackJeeves/ChocolatewithPepper'),
            cls('SmackJeeves/ClairetheFlare'),
            cls('SmackJeeves/ClockworkAtrium'),
            cls('SmackJeeves/CloeRemembrance'),
            cls('SmackJeeves/CockroachTheater'),
            cls('SmackJeeves/Cogs'),
            cls('SmackJeeves/ColorBlind'),
            cls('SmackJeeves/ConventionalWisdom'),
            cls('SmackJeeves/CosmicDash'),
            cls('SmackJeeves/Cramberries'),
            cls('SmackJeeves/CrimsonWings'),
            cls('SmackJeeves/CrocodileTears'),
            cls('SmackJeeves/CupOfOlea'),
            cls('SmackJeeves/CurseLineage'),
            cls('SmackJeeves/DanielleDark'),
            cls('SmackJeeves/Dasien'),
            cls('SmackJeeves/DavidDoesntGetIt'),
            cls('SmackJeeves/DBON'),
            cls('SmackJeeves/DeathNoteIridescent'),
            cls('SmackJeeves/DEGAF'),
            cls('SmackJeeves/DEMENTED'),
            cls('SmackJeeves/DemonBattles'),
            cls('SmackJeeves/DemonCat'),
            cls('SmackJeeves/DemonEater'),
            cls('SmackJeeves/DenizensAttention'),
            cls('SmackJeeves/DevilsCake'),
            cls('SmackJeeves/DevotoMusicinHell'),
            cls('SmackJeeves/Diaz'),
            cls('SmackJeeves/DigimonSaviors'),
            cls('SmackJeeves/DigimonTamersMiraiProject'),
            cls('SmackJeeves/DigitalInsanity'),
            cls('SmackJeeves/DoItYourself'),
            cls('SmackJeeves/DoodleBeans'),
            cls('SmackJeeves/DoodlingAround'),
            cls('SmackJeeves/Dragonet'),
            cls('SmackJeeves/DragonKid'),
            cls('SmackJeeves/DreamCatcher'),
            cls('SmackJeeves/DumpofManyPeople'),
            cls('SmackJeeves/DungeonHordes'),
            cls('SmackJeeves/EATATAU'),
            cls('SmackJeeves/EDepthAngel'),
            cls('SmackJeeves/EidolonWhispersOfEternity'),
            cls('SmackJeeves/ElementalSpirits'),
            cls('SmackJeeves/EnkeltenKentta'),
            cls('SmackJeeves/Enthrall'),
            cls('SmackJeeves/EntreEuxDeux'),
            cls('SmackJeeves/Eorah'),
            cls('SmackJeeves/Equsopia'),
            cls('SmackJeeves/ERAConvergence'),
            cls('SmackJeeves/ERAIbuki'),
            cls('SmackJeeves/ERRORERROR'),
            cls('SmackJeeves/EuphemisticEephus'),
            cls('SmackJeeves/EvilPlan'),
            cls('SmackJeeves/ExperimentalMegaman'),
            cls('SmackJeeves/EyesOfADigimon'),
            cls('SmackJeeves/FailureConfetti'),
            cls('SmackJeeves/FairyTaleRejects'),
            cls('SmackJeeves/FaithlessDigitals'),
            cls('SmackJeeves/FalconersDailyStrips'),
            cls('SmackJeeves/FallenAngelslove'),
            cls('SmackJeeves/FarOutMantic'),
            cls('SmackJeeves/FarOutThere'),
            cls('SmackJeeves/FeralGentry'),
            cls('SmackJeeves/FinalArcanum'),
            cls('SmackJeeves/FireredLisasReise'),
            cls('SmackJeeves/FlyorFail'),
            cls('SmackJeeves/ForcedSeduction'),
            cls('SmackJeeves/ForgetTheDistance'),
            cls('SmackJeeves/Fortheloveofabrokenstring'),
            cls('SmackJeeves/FramebyFrame'),
            cls('SmackJeeves/FrobertTheDemon'),
            cls('SmackJeeves/FromnowonImagirl'),
            cls('SmackJeeves/FruitloopAndMrDownbeat'),
            cls('SmackJeeves/FullSpectrumTherapy'),
            cls('SmackJeeves/GamerCafe'),
            cls('SmackJeeves/GamesPeoplePlayUpdatedWeekly'),
            cls('SmackJeeves/GardenofHearts'),
            cls('SmackJeeves/GayBacon'),
            cls('SmackJeeves/GayTimesWithRyanAndJay'),
            cls('SmackJeeves/GetUpAndGo'),
            cls('SmackJeeves/GigisNuzlockeRuns'),
            cls('SmackJeeves/Gloomverse'),
            cls('SmackJeeves/Gnoph'),
            cls('SmackJeeves/GoodGame'),
            cls('SmackJeeves/GoodnightMrsGoose'),
            cls('SmackJeeves/Grayscale'),
            cls('SmackJeeves/GuardiansoftheGalaxialSpaceways'),
            cls('SmackJeeves/Habibahssong'),
            cls('SmackJeeves/HarvestMoonParadiseFound'),
            cls('SmackJeeves/HateThePlayer'),
            cls('SmackJeeves/HatShop'),
            cls('SmackJeeves/Helix'),
            cls('SmackJeeves/HeltonShelton'),
            cls('SmackJeeves/Hephaestus'),
            cls('SmackJeeves/HereBeVoodoo'),
            cls('SmackJeeves/HiddenStrengthAWhiteNuzlocke'),
            cls('SmackJeeves/Hinata'),
            cls('SmackJeeves/Holocrash'),
            cls('SmackJeeves/HolyBlasphemy'),
            cls('SmackJeeves/HolyCrap'),
            cls('SmackJeeves/HopeForABreeze'),
            cls('SmackJeeves/HouseOfCraziness'),
            cls('SmackJeeves/HurrocksFardel'),
            cls('SmackJeeves/IciVontLesMorts'),
            cls('SmackJeeves/Inchoatica'),
            cls('SmackJeeves/Ingloriousbasterds'),
            cls('SmackJeeves/InHouseHumor'),
            cls('SmackJeeves/Inhuman'),
            cls('SmackJeeves/InsideOuTAYuriTale'),
            cls('SmackJeeves/InspiredByADream'),
            cls('SmackJeeves/ItsAn8BitWorldBlankWorld'),
            cls('SmackJeeves/IWishIggysWish'),
            cls('SmackJeeves/JackiesStory'),
            cls('SmackJeeves/Jantar'),
            cls('SmackJeeves/Jason'),
            cls('SmackJeeves/JoeysAdventure'),
            cls('SmackJeeves/JourneyMan'),
            cls('SmackJeeves/JoyToTheWorld'),
            cls('SmackJeeves/June'),
            cls('SmackJeeves/JustAnotherLife'),
            cls('SmackJeeves/JustCrazy'),
            cls('SmackJeeves/Justmyluck'),
            cls('SmackJeeves/KaitoShuno'),
            cls('SmackJeeves/KasaKeira'),
            cls('SmackJeeves/KazanatoFuneralPlanningService'),
            cls('SmackJeeves/KCNO'),
            cls('SmackJeeves/KezroChroniclesPhantomOps'),
            cls('SmackJeeves/Kirbandfriendsshowcase'),
            cls('SmackJeeves/KirbiesoftheAlternateDimension'),
            cls('SmackJeeves/KirbyAdventure'),
            cls('SmackJeeves/KirbyDreamTeam'),
            cls('SmackJeeves/KirbyFunfestTheOriginals'),
            cls('SmackJeeves/KirbysDreamAdventure'),
            cls('SmackJeeves/KirbysDreamlandAdventures'),
            cls('SmackJeeves/KirbyTheDeeArmy'),
            cls('SmackJeeves/KissmeSnow'),
            cls('SmackJeeves/KissoftheDevil'),
            cls('SmackJeeves/Knightface'),
            cls('SmackJeeves/KnightsRequiem'),
            cls('SmackJeeves/KojiX5'),
            cls('SmackJeeves/Kreetor'),
            cls('SmackJeeves/Kruptos'),
            cls('SmackJeeves/KuronaFlutterandLylaSpamTime'),
            cls('SmackJeeves/LastBlockStanding'),
            cls('SmackJeeves/LavenderLegend'),
            cls('SmackJeeves/LeCirquedObscure'),
            cls('SmackJeeves/LedbyaMadMan'),
            cls('SmackJeeves/LegendOfZeldaAHerosStory'),
            cls('SmackJeeves/LegendOfZeldaStaffOfPower'),
            cls('SmackJeeves/LegendOfZeldaTheEdgeAndTheLight'),
            cls('SmackJeeves/LegendOfZeldaTheWindWaker'),
            cls('SmackJeeves/Lemongrass'),
            cls('SmackJeeves/LesCendresdelHiver'),
            cls('SmackJeeves/LethalDose'),
            cls('SmackJeeves/LetLoveRule'),
            cls('SmackJeeves/LicensedHeroes'),
            cls('SmackJeeves/LifeAsACutOut'),
            cls('SmackJeeves/LifeAsItWas'),
            cls('SmackJeeves/LifeLessOrdinary'),
            cls('SmackJeeves/Lifeonpaper'),
            cls('SmackJeeves/LightLovers'),
            cls('SmackJeeves/LightwithinShadow'),
            cls('SmackJeeves/LilLevi'),
            cls('SmackJeeves/LOGOS'),
            cls('SmackJeeves/LOKI'),
            cls('SmackJeeves/LondonUnderworld'),
            cls('SmackJeeves/LostNova'),
            cls('SmackJeeves/LoveandIcecream'),
            cls('SmackJeeves/LoveHarbor'),
            cls('SmackJeeves/LoveMeLoveMyTeddyBear'),
            cls('SmackJeeves/LoveroftheSunandMoon'),
            cls('SmackJeeves/LsEmpire'),
            cls('SmackJeeves/LuffinpuffandEric'),
            cls('SmackJeeves/LumasParadise'),
            cls('SmackJeeves/MagicalMisfits'),
            cls('SmackJeeves/Magipunk'),
            cls('SmackJeeves/Manifestedpart1'),
            cls('SmackJeeves/MarioandLuigiMisadventures'),
            cls('SmackJeeves/MariosDayJob'),
            cls('SmackJeeves/MarioVsSonicVsMegaMan'),
            cls('SmackJeeves/MarsMind'),
            cls('SmackJeeves/MarXistemTWC'),
            cls('SmackJeeves/Mascara'),
            cls('SmackJeeves/MatildasSweetCakeCafe'),
            cls('SmackJeeves/MayTheRainCome'),
            cls('SmackJeeves/Mazscara'),
            cls('SmackJeeves/MegaManTales'),
            cls('SmackJeeves/MegaPain'),
            cls('SmackJeeves/MelodyAndMacabre'),
            cls('SmackJeeves/MetroJack'),
            cls('SmackJeeves/MidnightPrince'),
            cls('SmackJeeves/MineS'),
            cls('SmackJeeves/Minibot'),
            cls('SmackJeeves/MinorActsOfHeroism'),
            cls('SmackJeeves/Missing'),
            cls('SmackJeeves/Missingversionfrancaise'),
            cls('SmackJeeves/MobianChaos'),
            cls('SmackJeeves/Mokepon'),
            cls('SmackJeeves/Monstar'),
            cls('SmackJeeves/MoonValley'),
            cls('SmackJeeves/MorphE'),
            cls('SmackJeeves/Mortifer'),
            cls('SmackJeeves/MrFactory'),
            cls('SmackJeeves/MyFakeHeart'),
            cls('SmackJeeves/MySisterTheDragon'),
            cls('SmackJeeves/MySparklingPrincesama'),
            cls('SmackJeeves/MyStereoBot'),
            cls('SmackJeeves/MysticanDreams'),
            cls('SmackJeeves/MYth'),
            cls('SmackJeeves/MythsOfUnovaAWhiteNuzlockeRunHardMode'),
            cls('SmackJeeves/Nah'),
            cls('SmackJeeves/Negligence'),
            cls('SmackJeeves/NeonGlow'),
            cls('SmackJeeves/NeverTheHero'),
            cls('SmackJeeves/Nexus'),
            cls('SmackJeeves/NiceKitty'),
            cls('SmackJeeves/NighHeavenandHell'),
            cls('SmackJeeves/NightSpace'),
            cls('SmackJeeves/NIK'),
            cls('SmackJeeves/NissiesDragonPrincess'),
            cls('SmackJeeves/NixsFireRedNuzlocke'),
            cls('SmackJeeves/NobleHeartsHiruandMerroug'),
            cls('SmackJeeves/NoEnd'),
            cls('SmackJeeves/NormalcyisforWimps'),
            cls('SmackJeeves/NotyoursamI'),
            cls('SmackJeeves/ObnoxiousHerokun'),
            cls('SmackJeeves/ObsidianHeart'),
            cls('SmackJeeves/October20th'),
            cls('SmackJeeves/OddPlaceOddTime'),
            cls('SmackJeeves/OldElastikid'),
            cls('SmackJeeves/OneRainyDay'),
            cls('SmackJeeves/Onlyonelovesong'),
            cls('SmackJeeves/OperationTheater'),
            cls('SmackJeeves/OriginBook1Codearth'),
            cls('SmackJeeves/OurTimeInEden'),
            cls('SmackJeeves/Outbreak'),
            cls('SmackJeeves/OutofKey'),
            cls('SmackJeeves/OverSync'),
            cls('SmackJeeves/Panacea'),
            cls('SmackJeeves/PantsParty'),
            cls('SmackJeeves/PanzerDragonandEnigmaCompleteEdition'),
            cls('SmackJeeves/Pause'),
            cls('SmackJeeves/PencilviewUpdatesMondayscough'),
            cls('SmackJeeves/PeterPan'),
            cls('SmackJeeves/Phantomland'),
            cls('SmackJeeves/PhotoShootNarusasuDoujinshi'),
            cls('SmackJeeves/PlasticKings'),
            cls('SmackJeeves/PlayTime'),
            cls('SmackJeeves/PleaseBeMyBoytoy'),
            cls('SmackJeeves/PMDExplorersOfHeart'),
            cls('SmackJeeves/PMDTeamFirefox'),
            cls('SmackJeeves/PMDVictoryFire'),
            cls('SmackJeeves/PokemonBeta'),
            cls('SmackJeeves/PokemonCrystalDoubleNuzlockeChallenge'),
            cls('SmackJeeves/PokemonLANDSKY'),
            cls('SmackJeeves/PokemonNoRakuen'),
            cls('SmackJeeves/PokemonParallel'),
            cls('SmackJeeves/PokemonSAKOHJU'),
            cls('SmackJeeves/Pokeventurous'),
            cls('SmackJeeves/Ponzi'),
            cls('SmackJeeves/PrettyMouth'),
            cls('SmackJeeves/PrincessChroma'),
            cls('SmackJeeves/ProfessorDolphinpresentsPokemon'),
            cls('SmackJeeves/ProjectCAPLimit'),
            cls('SmackJeeves/PTO'),
            cls('SmackJeeves/Puck'),
            cls('SmackJeeves/PullingYouUnder'),
            cls('SmackJeeves/PulseandBolt'),
            cls('SmackJeeves/PurpureaNoxa'),
            cls('SmackJeeves/QueerQueen'),
            cls('SmackJeeves/RainbowMansion'),
            cls('SmackJeeves/RainLGBT'),
            cls('SmackJeeves/RainxSasori'),
            cls('SmackJeeves/RANDOM'),
            cls('SmackJeeves/RareCandyTreatment'),
            cls('SmackJeeves/RavenWolf'),
            cls('SmackJeeves/Regina'),
            cls('SmackJeeves/ReidyAndFriendsShowcase'),
            cls('SmackJeeves/RemoteAngel'),
            cls('SmackJeeves/Replica'),
            cls('SmackJeeves/Respectable'),
            cls('SmackJeeves/ReturntoEden'),
            cls('SmackJeeves/ROSIER'),
            cls('SmackJeeves/RottenApple'),
            cls('SmackJeeves/RoyalIcing'),
            cls('SmackJeeves/RubyNation'),
            cls('SmackJeeves/RuneSpark'),
            cls('SmackJeeves/RUScrewed'),
            cls('SmackJeeves/RyuManwebcomicVersion'),
            cls('SmackJeeves/SabishiiGhost'),
            cls('SmackJeeves/SaintforRent'),
            cls('SmackJeeves/SakuraDAY'),
            cls('SmackJeeves/SakuraMishzo'),
            cls('SmackJeeves/SalemUncommons'),
            cls('SmackJeeves/SallySprocketAndPistonPete'),
            cls('SmackJeeves/SaltyKiss'),
            cls('SmackJeeves/SayWhatYouMean'),
            cls('SmackJeeves/SChIzO'),
            cls('SmackJeeves/SchoolOfRejectsSoRe'),
            cls('SmackJeeves/ScionsoftheSeraph'),
            cls('SmackJeeves/ScrappedProject'),
            cls('SmackJeeves/SecretPowerbk1'),
            cls('SmackJeeves/SecretPowerbk2'),
            cls('SmackJeeves/Seki'),
            cls('SmackJeeves/SeriousTimes'),
            cls('SmackJeeves/SFCBlackjackBay'),
            cls('SmackJeeves/SFCForestofDreams'),
            cls('SmackJeeves/Shameless'),
            cls('SmackJeeves/ShamelessAdvertisements'),
            cls('SmackJeeves/ShotoutofCanon'),
            cls('SmackJeeves/ShroudofLight'),
            cls('SmackJeeves/Signifikat'),
            cls('SmackJeeves/SimpleBear'),
            cls('SmackJeeves/Sire'),
            cls('SmackJeeves/Skeptical'),
            cls('SmackJeeves/Slackmatic'),
            cls('SmackJeeves/SLightlyAbOVeAvErage'),
            cls('SmackJeeves/SlipstreamSingularity'),
            cls('SmackJeeves/SmallPressAdventures'),
            cls('SmackJeeves/SocksMittensandScarfs'),
            cls('SmackJeeves/SomethingLikeaPhenomenon'),
            cls('SmackJeeves/SonicAuthorAdventII'),
            cls('SmackJeeves/SonicBoom'),
            cls('SmackJeeves/SonicClub'),
            cls('SmackJeeves/SonicDashly'),
            cls('SmackJeeves/SonicFuture'),
            cls('SmackJeeves/SonicSchoolRedo'),
            cls('SmackJeeves/SOSRadio'),
            cls('SmackJeeves/SouthernCross'),
            cls('SmackJeeves/SovereignTheMostAmazingComicEver'),
            cls('SmackJeeves/SparElricsExtras'),
            cls('SmackJeeves/Spellcross'),
            cls('SmackJeeves/SpiderWings'),
            cls('SmackJeeves/SplitScreen'),
            cls('SmackJeeves/SPRITEDHeroesofDobalia'),
            cls('SmackJeeves/Spriterschaos'),
            cls('SmackJeeves/Sprytts'),
            cls('SmackJeeves/Stay'),
            cls('SmackJeeves/StellaInChrome'),
            cls('SmackJeeves/Stereophonic'),
            cls('SmackJeeves/StolenGeneration'),
            cls('SmackJeeves/Storyofadamnedlove'),
            cls('SmackJeeves/StrangersandFriends'),
            cls('SmackJeeves/Striped'),
            cls('SmackJeeves/StuntRayWalterswish'),
            cls('SmackJeeves/SubjecttoChangeCollegeWoes'),
            cls('SmackJeeves/Sunfall'),
            cls('SmackJeeves/SunmeetsMoon'),
            cls('SmackJeeves/SUNRISESTORY'),
            cls('SmackJeeves/SuperDimensionAfterTheHero'),
            cls('SmackJeeves/SuperMarioBros3'),
            cls('SmackJeeves/SuperMarjoBros'),
            cls('SmackJeeves/SupermassiveBlackHoleA'),
            cls('SmackJeeves/SurvivorFanCharacters'),
            cls('SmackJeeves/SweetestPoison'),
            cls('SmackJeeves/SwitchMechanism'),
            cls('SmackJeeves/TaikiTheWebcomic'),
            cls('SmackJeeves/TailsAdventureThroughTimeandOtherWorlds'),
            cls('SmackJeeves/TakingPicturesofStrangers'),
            cls('SmackJeeves/TalesFromAaronsWings'),
            cls('SmackJeeves/TEN'),
            cls('SmackJeeves/ThatWasntThereYesterday'),
            cls('SmackJeeves/The13thWorld'),
            cls('SmackJeeves/TheAdventuresOfBanjoZ'),
            cls('SmackJeeves/TheAntihero'),
            cls('SmackJeeves/TheArchipelago'),
            cls('SmackJeeves/Theatrics'),
            cls('SmackJeeves/TheBattleInTheSky'),
            cls('SmackJeeves/TheBookOfNosferatu'),
            cls('SmackJeeves/TheBrideoftheShark'),
            cls('SmackJeeves/TheBucket'),
            cls('SmackJeeves/TheCafedAlizee'),
            cls('SmackJeeves/TheCavernOfSecrets'),
            cls('SmackJeeves/TheColony'),
            cls('SmackJeeves/TheContract'),
            cls('SmackJeeves/TheCrawl'),
            cls('SmackJeeves/TheDarkLegacy'),
            cls('SmackJeeves/TheDemonicAdventuresOfAngelWitchPita'),
            cls('SmackJeeves/TheDestroyer'),
            cls('SmackJeeves/TheDragonAndTheLemur'),
            cls('SmackJeeves/TheDreaming'),
            cls('SmackJeeves/TheDrifter'),
            cls('SmackJeeves/TheElectricRose'),
            cls('SmackJeeves/TheForestofWhispers'),
            cls('SmackJeeves/TheGhostWithTheMost'),
            cls('SmackJeeves/TheGoldRiderofPern'),
            cls('SmackJeeves/TheGrayZone'),
            cls('SmackJeeves/TheHeadhunters'),
            cls('SmackJeeves/TheHeartofEarth'),
            cls('SmackJeeves/TheiaMania'),
            cls('SmackJeeves/TheJosephComics'),
            cls('SmackJeeves/TheKeyHotelEnding'),
            cls('SmackJeeves/TheKwiddexProtocol'),
            cls('SmackJeeves/TheLastBloodCafe'),
            cls('SmackJeeves/ThelaughingDeath'),
            cls('SmackJeeves/TheLegendaryQueen'),
            cls('SmackJeeves/TheLifeofMagFlamequill'),
            cls('SmackJeeves/TheLoneSwordsman'),
            cls('SmackJeeves/TheMadMan'),
            cls('SmackJeeves/TheMegaManandSonicSpriteShowcase'),
            cls('SmackJeeves/TheNightSurfers'),
            cls('SmackJeeves/TheNocheComicSeries'),
            cls('SmackJeeves/TheNOMEDSEGA'),
            cls('SmackJeeves/ThePirateBalthasar'),
            cls('SmackJeeves/ThePremise'),
            cls('SmackJeeves/ThePrincessAndTheGiant'),
            cls('SmackJeeves/ThePropertyofHate'),
            cls('SmackJeeves/TheReborn'),
            cls('SmackJeeves/TheSearchForHenryJekyll'),
            cls('SmackJeeves/TheSilverLeague'),
            cls('SmackJeeves/TheSummerofBlakeSinclair'),
            cls('SmackJeeves/Theswordsmanandtheamnesiac'),
            cls('SmackJeeves/TheTimeDog'),
            cls('SmackJeeves/TheTytonNuzlockeChallengeEmeraldEdition'),
            cls('SmackJeeves/TheWhiteTower'),
            cls('SmackJeeves/TheWinterCampaign'),
            cls('SmackJeeves/TheYoshiHerd'),
            cls('SmackJeeves/ThiefCatcherRingTail'),
            cls('SmackJeeves/ThornsComic'),
            cls('SmackJeeves/ThornTopia'),
            cls('SmackJeeves/TLAAOK'),
            cls('SmackJeeves/ToddAllisonAndThePetuniaViolet'),
            cls('SmackJeeves/TosiHuonoYaoiSarjis'),
            cls('SmackJeeves/TotallyCrossover'),
            cls('SmackJeeves/TPTruePower'),
            cls('SmackJeeves/TrainerWantsToFight'),
            cls('SmackJeeves/Transfusions'),
            cls('SmackJeeves/TransUMan'),
            cls('SmackJeeves/TroubleNextDoor'),
            cls('SmackJeeves/UglyBoysLove'),
            cls('SmackJeeves/Uglygame'),
            cls('SmackJeeves/UnderTheDeadSkies'),
            cls('SmackJeeves/UnicampaLapis'),
            cls('SmackJeeves/UpDown'),
            cls('SmackJeeves/UshalaatWorldsEnd'),
            cls('SmackJeeves/Vacan7'),
            cls('SmackJeeves/VACANT'),
            cls('SmackJeeves/VerloreGeleentheid'),
            cls('SmackJeeves/VoidMisadventures'),
            cls('SmackJeeves/VoyageoftheBrokenPromise'),
            cls('SmackJeeves/WakeEcho'),
            cls('SmackJeeves/Wander'),
            cls('SmackJeeves/WantedDeadorDead'),
            cls('SmackJeeves/Wayfar'),
            cls('SmackJeeves/Waysoftheheart'),
            cls('SmackJeeves/WeAreGolden'),
            cls('SmackJeeves/WelcomeToFreakshow'),
            cls('SmackJeeves/WelcomeToThePCA'),
            cls('SmackJeeves/WhatAboutLove'),
            cls('SmackJeeves/WHATAboutSHADOWS'),
            cls('SmackJeeves/WhatIsDeepInOnesHeart'),
            cls('SmackJeeves/WhatWeRememberTheMost'),
            cls('SmackJeeves/WhenSheWasBad'),
            cls('SmackJeeves/Whenweweresilent'),
            cls('SmackJeeves/WhereaboutsOfTime'),
            cls('SmackJeeves/WhiteHeart'),
            cls('SmackJeeves/WhiteNoise'),
            cls('SmackJeeves/Wildflowers'),
            cls('SmackJeeves/WildWingBoys'),
            cls('SmackJeeves/WildWingBoysKoathArc'),
            cls('SmackJeeves/WingsOverEthereal'),
            cls('SmackJeeves/WingsTurnedtoDust'),
            cls('SmackJeeves/Wootlabs'),
            cls('SmackJeeves/XXMoralityXx'),
            cls('SmackJeeves/YadotCakeShop'),
            cls('SmackJeeves/YamanaokiHighSchool'),
            cls('SmackJeeves/YoungCannibals'),
            cls('SmackJeeves/ZaenWell'),
            cls('SmackJeeves/ZeldaTheNewAdventureofLinkIIMajorasMask'),
            cls('SnafuComics/KOF'),
            cls('SnafuComics/MyPanda'),
            cls('SnafuComics/SF'),
            cls('SnafuComics/Snafu'),
            cls('SnafuComics/Tin'),
            cls('SnafuComics/Titan'),
            cls('StudioKhimera/Eorah', 'mov'),
            cls('StuffNoOneToldMe'),
            cls('TaleOfTenThousand'),
            cls('TheCyantianChronicles/CookieCaper'),
            cls('TheCyantianChronicles/Pawprints'),
            cls('VGCats/Adventure'),
            cls('VGCats/Super'),
            cls('VictimsOfTheSystem'),
            cls('WebDesignerCOTW'),
            cls('WebToons/Adamsville'),
            cls('WebToons/CrapIDrewOnMyLunchBreak'),
            cls('WintersLight'),

            # Removed in 3.1
            cls('AbbysAgency', 'brk'),
            cls('AcademyVale'),
            cls('AhoyEarth', 'block'),
            cls('Anaria', 'del'),
            cls('Angels2200', 'del'),
            cls('BlackRose', 'brk'),
            cls('BloodBound', 'deny'),
            cls('CatenaManor/CatenaCafe'),
            cls('ComicsKingdom/AmazingSpiderman'),
            cls('ComicsKingdom/AmazingSpidermanSpanish'),
            cls('ComicsKingdom/BigBenBoltSundays'),
            cls('ComicsKingdom/BonersArkSundays'),
            cls('ComicsKingdom/BrianDuffy'),
            cls('ComicsKingdom/Crankshaft'),
            cls('ComicsKingdom/FlashGordonSundays'),
            cls('ComicsKingdom/FunkyWinkerbean'),
            cls('ComicsKingdom/FunkyWinkerbeanSunday'),
            cls('ComicsKingdom/FunkyWinkerbeanSundays'),
            cls('ComicsKingdom/FunkyWinkerbeanVintage'),
            cls('ComicsKingdom/HeartOfJulietJonesSundays'),
            cls('ComicsKingdom/KatzenjammerKidsSundays'),
            cls('ComicsKingdom/Lockhorns'),
            cls('ComicsKingdom/MandrakeTheMagicianSundays'),
            cls('ComicsKingdom/MikePeters'),
            cls('ComicsKingdom/MotherGooseAndGrimm'),
            cls('ComicsKingdom/PhantomSundays'),
            cls('ComicsKingdom/PrinceValiantSundays'),
            cls('ComicsKingdom/Retail'),
            cls('ComicsKingdom/TigerSundays'),
            cls('Everblue', 'block'),
            cls('FalseStart'),
            cls('FireflyCross'),
            cls('Ginpu'),
            cls('GoComics/9ChickweedLaneClassics'),
            cls('GoComics/Badlands'),
            cls('GoComics/BeanieTheBrownie'),
            cls('GoComics/BearWithMe'),
            cls('GoComics/BigNateFirstClass'),
            cls('GoComics/BreakOfDay'),
            cls('GoComics/Candorville'),
            cls('GoComics/DilbertClassics'),
            cls('GoComics/DilbertEnEspanol'),
            cls('GoComics/DumbwichCastle'),
            cls('GoComics/EyebeamClassic'),
            cls('GoComics/GarfieldClassics'),
            cls('GoComics/LaloAlcarazEnEspanol'),
            cls('GoComics/MakingIt'),
            cls('GoComics/MtPleasant'),
            cls('GoComics/PCAndPixel'),
            cls('GoComics/PetuniaAndDre'),
            cls('GoComics/RosebudsEnEspanol'),
            cls('GoComics/RudyPark'),
            cls('GoComics/SaltNPepper'),
            cls('GoComics/SigneWilkinson'),
            cls('GoComics/Snowflakes'),
            cls('GoComics/StoneSoupClassics'),
            cls('GoComics/StuartCarlson'),
            cls('GoComics/SunshineState'),
            cls('GoComics/TruthFacts'),
            cls('GoComics/Wannabe'),
            cls('KemonoCafe/PrincessBunny'),
            cls('Lackadaisy', 'block'),
            cls('MangaDex/AttackOnTitan', 'legal'),
            cls('MangaDex/DeliciousinDungeon', 'legal'),
            cls('MangaDex/FuguushokuKajishiDakedoSaikyouDesu', 'legal'),
            cls('MangaDex/ImTheMaxLevelNewbie', 'legal'),
            cls('MangaDex/KaetteKudasaiAkutsuSan', 'legal'),
            cls('MangaDex/PashiriNaBokuToKoisuruBanchouSan', 'legal'),
            cls('MangaDex/PleaseDontBullyMeNagatoro', 'legal'),
            cls('MangaDex/SaekiSanWaNemutteru', 'legal'),
            cls('MangaDex/SoloLeveling', 'legal'),
            cls('MangaDex/SousouNoFrieren', 'legal'),
            cls('MangaDex/TenseiShitaraSlimeDattaKen', 'legal'),
            cls('MangaDex/YuYuHakusho', 'legal'),
            cls('MrLovenstein', 'jsh'),
            cls('MyCartoons'),
            cls('RickGriffinStudios/TracesOfThePastNSFW'),
            cls('Shivae/BlackRose', 'brk'),
            cls('SoloLeveling', 'legal'),
            cls('StudioKhimera/Mousechevious'),
            cls('TalesAndTactics'),
            cls('Tapas/HoneyAndTheMoon'),
            cls('TracesOfThePast/NSFW'),
            cls('VampireHunterBoyfriends'),
            cls('WebToons/ABudgiesLife'),
            cls('WebToons/Anthronauts'),
            cls('WebToons/AssassinRoommate'),
            cls('WebToons/ChocoLatte'),
            cls('WebToons/CrystalVirus'),
            cls('WebToons/NightmareFactory'),
            cls('WebToons/OVERPOWERED'),

            # Removed in 3.2
            cls('ComicsKingdom/BeetleMoses'),
            cls('ComicsKingdom/BobMankoffPresentsShowMeTheFunny'),
            cls('ComicsKingdom/BobMankoffPresentsShowMeTheFunnyAnimalEdition'),
            cls('ComicsKingdom/Candorville'),
            cls('ComicsKingdom/DarrinBell'),
            cls('ComicsKingdom/FunnyOnlineAnimals'),
            cls('ComicsKingdom/GodsHands'),
            cls('ComicsKingdom/MaraLlaveKeeperOfTime'),
            cls('Dilbert'),
            cls('GoComics/DarrinBell'),
            cls('GoComics/EverydayPeopleCartoons'),
        )


class Renamed(Scraper):
    MSG = 'Comic module was renamed/moved to "%s", please use the new name instead.'
    count = 0

    @classmethod
    def counter(cls):
        cls.count += 1
        return cls.count

    def __init__(self, name, newname):
        super(Renamed, self).__init__(name)
        self.newname = newname
        self.i = self.counter()

    def getDisabledReasons(self):
        return {'ren-%i' % self.i: self.MSG % self.newname}

    @classmethod
    def getmodules(cls):
        return (
            # Renamed in 2.16
            cls('1997', '1977'),
            cls('ApartmentForTwo', 'NamirDeiter/ApartmentForTwo'),
            cls('Catena', 'CatenaManor'),
            cls('ComicFury/Alya', 'ComicFury/AlyaTheLastChildOfLight'),
            cls('ComicFury/Boatcrash', 'ComicFury/BoatcrashChronicles'),
            cls('ComicFury/Crimsonpixel', 'ComicFury/CrimsonPixelComics'),
            cls('ComicFury/Doublejump', 'ComicFury/DoubleJumpGameComics'),
            cls('ComicFury/Elektroanthology', 'ComicFury/ElektrosComicAnthology'),
            cls('ComicFury/ICanSeeYourFeels', 'ComicFury/SeeYourFeels'),
            cls('ComicFury/MAGISAupdatesMonWedFri', 'ComicFury/MAGISAPARASAYOupdatesMonFri'),
            cls('ComicFury/MonsterSoup', 'MonsterSoup'),
            cls('ComicFury/OopsComicAdventure', 'OopsComicAdventure'),
            cls('ComicFury/ThomasAndZachary', 'ComicFury/ThomasAndZacharyArchives'),
            cls('ComicGenesis/TheLounge', 'KeenSpot/TheLounge'),
            cls('Creators/ArchieinSpanish', 'Creators/ArchieSpanish'),
            cls('Creators/HeathcliffinSpanish', 'Creators/HeathcliffSpanish'),
            cls('Creators/TheWizardofIdinSpanish', 'Creators/WizardOfIdSpanish'),
            cls('DarkWings', 'Eryl'),
            cls('EyeOfRamalach', 'KemonoCafe/TheEyeOfRamalach'),
            cls('FoulLanguage', 'GoComics/FowlLanguage'),
            cls('KeenSpot/AntiheroForHire', 'AntiheroForHire'),
            cls('KeenSpot/ElGoonishShive', 'ElGoonishShive'),
            cls('KeenSpot/ElGoonishShiveNP', 'ElGoonishShiveNP'),
            cls('KeenSpot/Newshounds', 'Newshounds'),
            cls('KeenSpot/SinFest', 'SinFest'),
            cls('KeenSpot/TheGodChild', 'GodChild'),
            cls('LasLindas', 'KemonoCafe/LasLindas'),
            cls('NicoleAndDerek', 'NamirDeiter/NicoleAndDerek'),
            cls('OnTheFasttrack', 'ComicsKingdom/OnTheFastrack'),
            cls('PetiteSymphony/Djandora', 'ComicsBreak/Djandora'),
            cls('PetiteSymphony/Generation17', 'ComicsBreak/Generation17'),
            cls('PetiteSymphony/Rascals', 'KemonoCafe/Rascals'),
            cls('QuentynQuinnSpaceRanger', 'RHJunior/QuentynQuinnSpaceRanger'),
            cls('ShermansLagoon', 'GoComics/ShermansLagoon'),
            cls('SmackJeeves/AddictiveScience', 'KemonoCafe/AddictiveScience'),
            cls('SmackJeeves/CityFolk', 'ComicFury/CityFolk'),
            cls('SmackJeeves/DoomsdayMyDear', 'DoomsdayMyDear'),
            cls('SmackJeeves/ForestHill', 'ForestHill'),
            cls('SmackJeeves/Katran', 'ComicFury/KATRAN'),
            cls('SmackJeeves/LatchkeyKingdom', 'ComicFury/LatchkeyKingdom'),
            cls('SmackJeeves/Magience', 'ComicFury/Magience'),
            cls('SmackJeeves/RiversideExtras', 'RiversideExtras'),
            cls('SmackJeeves/StarTrip', 'StarTrip'),
            cls('TalesOfTheQuestor', 'RHJunior/TalesOfTheQuestor'),
            cls('TheProbabilityBomb', 'RHJunior/TheProbabilityBomb'),
            cls('TracyAndTristan', 'ComicFury/TracyAndTristan'),
            cls('UnlikeMinerva', 'NamirDeiter/UnlikeMinerva'),
            cls('Wulffmorgenthaler', 'WuMo'),
            cls('YouSayItFirst', 'NamirDeiter/YouSayItFirst'),
            cls('ZebraGirl', 'ComicFury/ZebraGirl'),

            # Renamed in 3.0
            cls('AHClub', 'RickGriffinStudios/AHClub'),
            cls('ComicFury/MuddlemarchMudCompany', 'ComicFury/MudCompany'),
            cls('ComicsKingdom/ShermansLagoon', 'GoComics/ShermansLagoon'),
            cls('ComicsKingdom/TheLittleKing', 'ComicsKingdom/LittleKing'),
            cls('GoComics/BloomCounty2017', 'GoComics/BloomCountyContinues'),
            cls('GoComics/Owlturd', 'GoComics/ShenComix'),
            cls('GoComics/RipleysBelieveItOrNotSpanish', 'GoComics/RipleysAunqueUstedNoLoCrea'),
            cls('GoComics/WebcomicName', 'WebcomicName'),
            cls('GoComics/Widdershins', 'Widdershins'),
            cls('Guardia', 'ComicFury/Guardia'),
            cls('RadioactivePanda', 'Tapas/RadioactivePanda'),
            cls('SmackJeeves/BlackTapestries', 'ComicFury/BlackTapestries'),
            cls('SmackJeeves/ByTheBook', 'ByTheBook'),
            cls('SmackJeeves/FurryExperience', 'ComicFury/FurryExperience'),
            cls('SmackJeeves/GrowingTroubles', 'ComicFury/GrowingTroubles'),
            cls('SmackJeeves/TheRealmOfKaerwyn', 'ComicFury/TheRealmOfKaerwyn'),
            cls('StudioKhimera/Draconia', 'Draconia'),
            cls('StudioKhimera/UberQuest', 'UberQuest'),
            cls('TracesOfThePast', 'RickGriffinStudios/TracesOfThePast'),

            # Renamed in 3.1
            cls('ComicsKingdom/SlylockFoxAndComicsForKids', 'ComicsKingdom/SlylockFox'),
            cls('ComicsKingdom/SlylockFoxAndComicsForKidsSpanish',
                'ComicsKingdom/SlylockFoxSpanish'),
            cls('Derideal/LRE', 'Derideal/RLE'),
            cls('Exiern', 'ComicFury/Exiern'),
            cls('GoComics/FalseKnees', 'WebToons/FalseKnees'),
            cls('GoComics/HeavenlyNostrils', 'GoComics/PhoebeAndHerUnicorn'),
            cls('GoComics/Rosebuds', 'ComicsKingdom/Rosebuds'),
            cls('MangaDex/TheWolfAndRedRidingHood', 'WebToons/TheWolfAndRedRidingHood'),
            cls('MaxOveracts', 'OccasionalComicsDisorder'),
            cls('SafelyEndangered', 'WebToons/SafelyEndangered'),

            # Renamed in 3.2
            cls('ComicsKingdom/WillyBlacksSpanish', 'ComicsKingdom/WillyBlackSpanish'),
            cls('GoComics/BloomCounty2019', 'GoComics/BloomCountyContinues'),
            cls('GoComics/CathyClassics', 'GoComics/Cathy'),
            cls('GoComics/OutOfTheGenePoolReRuns', 'GoComics/OutOfTheGenePool'),
            cls('GoComics/SnoopyEnEspanol', 'GoComics/PeanutsEnEspanol'),
            cls('GoComics/TheWizardOfIdSpanish', 'GoComics/WizardOfIdEnEspanol'),
            cls('PiledHigherAndDeeper', 'PHDComics'),
        )



# SPDX-License-Identifier: MIT
# ISO 693-1 language codes from pycountry
# This file is automatically generated, DO NOT EDIT!
Languages = {
    'de': 'German',
    'en': 'English',
    'es': 'Spanish',
    'fi': 'Finnish',
    'fr': 'French',
    'it': 'Italian',
    'ja': 'Japanese',
    'pt': 'Portuguese',
}



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from re import compile, escape

from ..scraper import BasicScraper
from ..util import tagre
from ..helpers import indirectStarter
from .common import ComicControlScraper


class JackCannon(BasicScraper):
    url = 'http://fancyadventures.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2008/07/07/2008-07-08'
    imageSearch = compile(tagre("img", "src", r'(%scomics/[^"]+)' % rurl))
    prevSearch = compile(tagre("a", "href", r'(%s[^"]+)' % rurl, after="prev"))
    help = 'Index format: yyyy/mm/dd/page-nnn'


class JimBenton(BasicScraper):
    url = 'http://www.jimbenton.com/page14/page14.html'
    stripUrl = 'http://www.jimbenton.com/page14/files/JimBentonComic-%s.html'
    starter = indirectStarter
    imageSearch = compile(tagre("img", "src", r'(JimBentonComic-[^"]+)',
                                before="photo-frame"))
    prevSearch = compile(tagre("a", "href", r'(JimBentonComic-[^>]+\.html)',
                               quote="") + "Next")
    latestSearch = compile(tagre("a", "href", r'(files/JimBentonComic-[^>]+\.html)', quote=""))
    help = 'Index format: stripname'


class JoeAndMonkey(BasicScraper):
    url = 'http://www.joeandmonkey.com/'
    stripUrl = url + '%s'
    imageSearch = compile(r'"(/comic/[^"]+)"')
    prevSearch = compile(r"<a href='(/\d+)'>Previous")
    help = 'Index format: nnn'


class JohnnyWander(ComicControlScraper):
    imageSearch = ('//ul[d:class("cc-showbig")]/li/@data-src',
        '//img[@id="cc-comic"]')
    url = 'http://www.johnnywander.com/'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Thomas W. Littauer
from __future__ import annotations

from .. import scraper
from ..helpers import joinPathPartsNamer


def fixlink(url: str) -> str:
    return url.replace('//wp.', '//', 1)


class ComicsKingdom(scraper.Scraper):
    namer = joinPathPartsNamer(pageparts=range(-2, 0))
    help = 'Index format: yyyy-mm-dd'
    prevSearch = 'prev'
    imageSearch = 'img'

    def __init__(self, name: str, path: str, lang: str | None = None) -> None:
        super().__init__('ComicsKingdom/' + name)
        self.url = 'https://comicskingdom.com/' + path
        self.stripUrl = self.url + '/%s'
        self.path = path
        if lang:
            self.lang = lang
        self.data: dict[str, dict[str, str]] = {}
        self.nextapi: str | None = None
        self.lastseen: str | None = None
        self.latest: str | None = None

    def starter(self):
        self.fetchmore()
        return self.latest

    def getPage(self, url):
        while 'prev' not in self.data.get(url, {}) and self.nextapi:
            self.fetchmore()
        return self.data[url]

    def fetchmore(self) -> None:
        """Fetch one page from the API into our cache."""
        if not self.nextapi:
            r = self.session.get('https://wp.comicskingdom.com/wp-json/wp/v2/ck_comic', params={
                'ck_feature': self.path,
                'per_page': '75',
                '_fields': 'link,title,assets,type',
            })
        else:
            r = self.session.get(self.nextapi)
        r.raise_for_status()
        self.nextapi = r.links['next']['url'] if 'next' in r.links else None
        newdata = r.json()
        for page in newdata:
            url = fixlink(page['link'])
            self.data[url] = {
                'img': page['assets']['single']['url'],
            }
            if not self.lastseen:
                self.latest = url
            else:
                self.data[self.lastseen]['prev'] = url
            self.lastseen = url
        if not self.nextapi:
            self.firstStripUrl = self.lastseen

    def fetchUrls(self, url, data, urlsearch):
        # HACK: Just get the correct data from the cached JSON
        return (data[urlsearch],)

    @classmethod
    def getmodules(cls):
        return (
            # do not edit anything below since these entries are generated from
            # scripts/comicskingdom.py
            # START AUTOUPDATE
            cls('Alice', 'alice'),
            cls('AmberWaves', 'amber-waves'),
            cls('Apartment3G', 'apartment-3-g_1'),
            cls('ArcticCircle', 'arctic-circle'),
            cls('ATodaVelocidadSpanish', 'a-toda-velocidad', lang='es'),
            cls('BarneyGoogleAndSnuffySmith', 'barney-google-and-snuffy-smith'),
            cls('BarneyGoogleAndSnuffySmithSpanish', 'tapon', lang='es'),
            cls('BarneyGoogleAndSnuffySmithVintage', 'barney-google-and-snuffy-smith-vintage'),
            cls('BeetleBailey', 'beetle-bailey-1'),
            cls('BeetleBaileySpanish', 'beto-el-recluta', lang='es'),
            cls('BeetleBaileyVintage', 'beetle-bailey-vintage'),
            cls('BetweenFriends', 'between-friends'),
            cls('BewareOfToddler', 'beware-of-toddler'),
            cls('BigBenBolt', 'big-ben-bolt'),
            cls('Bizarro', 'bizarro'),
            cls('Blondie', 'blondie'),
            cls('BlondieSpanish', 'pepita', lang='es'),
            cls('BonersArk', 'boners-ark'),
            cls('BreakOfDay', 'break-of-day'),
            cls('BrickBradford', 'brick-bradford'),
            cls('BrilliantMindOfEdisonLee', 'brilliant-mind-of-edison-lee'),
            cls('BringingUpFather', 'bringing-up-father'),
            cls('BringingUpFatherSpanish', 'educando-a-papa', lang='es'),
            # BroomHilda has a duplicate in GoComics/BroomHilda
            cls('BuzSawyer', 'buz-sawyer'),
            cls('CarpeDiem', 'carpe-diem'),
            cls('Comiclicious', 'comiclicious'),
            cls('Crock', 'crock'),
            cls('CrockSpanish', 'crock-spanish', lang='es'),
            cls('Curtis', 'curtis'),
            cls('DaddyDaze', 'daddy-daze'),
            cls('DavidMHitch', 'david-m-hitch'),
            cls('DennisTheMenace', 'dennis-the-menace'),
            cls('DennisTheMenaceSpanish', 'daniel-el-travieso', lang='es'),
            # DickTracy has a duplicate in GoComics/DickTracy
            cls('Dumplings', 'dumplings'),
            cls('Dustin', 'dustin'),
            cls('EdGamble', 'ed-gamble'),
            # EdgeCity has a duplicate in GoComics/EdgeCity
            cls('FamilyCircus', 'family-circus'),
            cls('FamilyCircusSpanish', 'circulo-familiar', lang='es'),
            cls('FlashForward', 'flash-forward'),
            cls('FlashGordon', 'flash-gordon'),
            cls('FlashGordonVintage', 'flash-gordon-vintage'),
            cls('FlashGordonVintageSundays', 'flash-gordon-vintage-sunday'),
            cls('GearheadGertie', 'gearhead-gertie'),
            cls('Goomer', 'goomer'),
            cls('GoomerSpanish', 'goomer-spanish', lang='es'),
            cls('HagarTheHorrible', 'hagar-the-horrible'),
            cls('HagarTheHorribleSpanish', 'olafo', lang='es'),
            cls('HeartOfJulietJones', 'heart-of-juliet-jones'),
            cls('HiAndLois', 'hi-and-lois'),
            cls('HiAndLoisVintage', 'hi-and-lois-vintage'),
            cls('InsanityStreak', 'insanity-streak'),
            cls('IntelligentLife', 'intelligent'),
            cls('JimmyMargulies', 'jimmy-margulies'),
            cls('JohnBranch', 'john-branch'),
            cls('JohnnyHazard', 'johnny-hazard'),
            cls('JudgeParker', 'judge-parker'),
            cls('JudgeParkerVintage', 'judge-parker-vintage'),
            cls('JungleJimSundays', 'jungle-jim-sundays'),
            cls('KatzenjammerKids', 'katzenjammer-kids'),
            cls('KatzenjammerKidsSpanish', 'maldades-de-dos-pilluelos', lang='es'),
            cls('KatzenjammerKidsVintageSundays', 'katzenjammer-kids-vintage-sunday'),
            cls('KevinAndKell', 'kevin-and-kell'),
            cls('KingOfTheRoyalMounted', 'king-of-the-royal-mounted'),
            cls('KirkWalters', 'kirk-walters'),
            cls('KrazyKat', 'krazy-kat'),
            cls('LaloYLolaSpanish', 'lalo-y-lola', lang='es'),
            cls('LeeJudge', 'lee-judge'),
            cls('LegalizationNation', 'legalization-nation'),
            cls('LegendOfBill', 'legend-of-bill'),
            cls('LittleIodineSundays', 'little-iodine-sundays'),
            cls('LittleKing', 'the-little-king'),
            cls('Macanudo', 'macanudo'),
            cls('MacanudoSpanish', 'macanudo-spanish', lang='es'),
            cls('MallardFillmore', 'mallard-fillmore'),
            cls('MandrakeTheMagician', 'mandrake-the-magician'),
            cls('MandrakeTheMagicianSpanish', 'mandrake-the-magician-spanish', lang='es'),
            cls('MandrakeTheMagicianVintage', 'mandrake-the-magician-vintage'),
            cls('MandrakeTheMagicianVintageSundays', 'mandrake-the-magician-vintage-sunday'),
            cls('MarkTrail', 'mark-trail'),
            cls('MarkTrailSpanish', 'mark-trail-spanish', lang='es'),
            cls('MarkTrailVintage', 'mark-trail-vintage'),
            cls('Marvin', 'marvin'),
            cls('MarvinSpanish', 'marvin-spanish', lang='es'),
            cls('MaryWorth', 'mary-worth'),
            cls('MaryWorthSpanish', 'maria-de-oro', lang='es'),
            cls('Mazetoons', 'mazetoons'),
            cls('MikeShelton', 'mike-shelton'),
            cls('MikeSmith', 'mike-smith'),
            cls('MooseAndMolly', 'moose-and-molly'),
            cls('MooseAndMollySpanish', 'quintin', lang='es'),
            cls('MrAbernathySpanish', 'don-abundio', lang='es'),
            cls('Mutts', 'mutts'),
            cls('MuttsSpanish', 'motas', lang='es'),
            cls('NeverBeenDeader', 'never-been-deader'),
            cls('NewTricks', 'new-tricks'),
            cls('NibblesAndScratch', 'nibbles-scratch'),
            cls('OddlyEnough', 'oddly-enough'),
            cls('OfficeHours', 'office-hours'),
            cls('OliveAndPopeye', 'olive-popeye'),
            cls('OnTheFastrack', 'on-the-fastrack'),
            cls('PajamaDiaries', 'pajama-diaries'),
            cls('Palurdeando', 'palurdeando'),
            cls('PalurdeandoSpanish', 'palurdeando-spanish', lang='es'),
            cls('PardonMyPlanet', 'pardon-my-planet'),
            cls('Phantom', 'phantom'),
            cls('PhantomSpanish', 'el-fantasma', lang='es'),
            cls('PhantomVintage', 'the-phantom-vintage'),
            cls('PhantomVintageSundays', 'the-phantom-vintage-sunday'),
            cls('PlanetSyndicate', 'the-planet-syndicate'),
            # Pluggers has a duplicate in GoComics/Pluggers
            cls('Popeye', 'popeye'),
            cls('PopeyesCartoonClub', 'popeyes-cartoon-club'),
            cls('PopeyeSpanish', 'popeye-spanish', lang='es'),
            cls('PrinceValiant', 'prince-valiant'),
            cls('PrinceValiantVintageSundays', 'prince-valiant-vintage-sunday'),
            cls('PrincipeValienteSpanish', 'principe-valiente', lang='es'),
            cls('ProsAndCons', 'pros-cons'),
            cls('Quincy', 'quincy'),
            cls('RadioPatrol', 'radio-patrol'),
            cls('RaeTheDoe', 'rae-the-doe'),
            cls('RexMorganMD', 'rex-morgan-m-d'),
            cls('RexMorganMDSpanish', 'rex-morgan-md-spanish', lang='es'),
            cls('RhymesWithOrange', 'rhymes-with-orange'),
            cls('RipKirby', 'rip-kirby'),
            cls('Rosebuds', 'rosebuds'),
            cls('SafeHavens', 'safe-havens'),
            cls('SagaOfBrannBjornson', 'the-saga-of-brann-bjornson'),
            cls('Sales', 'sales'),
            cls('SallyForth', 'sally-forth'),
            cls('SamAndSilo', 'sam-and-silo'),
            cls('SamAndSiloSpanish', 'soso-y-siso', lang='es'),
            cls('SecretAgentX9', 'secret-agent-x-9'),
            # Shoe has a duplicate in GoComics/Shoe
            cls('SixChix', 'six-chix'),
            cls('SlylockFox', 'slylock-fox-and-comics-for-kids'),
            cls('SlylockFoxSpanish', 'solo-para-ninos', lang='es'),
            cls('SuburbanFairyTales', 'suburban-fairy-tales'),
            cls('TakeItFromTheTinkersons', 'take-it-from-the-tinkersons'),
            cls('TBDToonsByDan', 'tbd-toons-by-dan'),
            cls('TheyllDoItEveryTimeSpanish', 'nunca-falta-alguien-asi', lang='es'),
            cls('ThimbleTheater', 'thimble-theater'),
            cls('Tiger', 'tiger'),
            cls('TigerSpanish', 'tigrillo', lang='es'),
            cls('TigerVintage', 'tiger-vintage'),
            cls('TigerVintageSundays', 'tiger-vintage-sunday'),
            cls('TinasGroove', 'tina-s-groove'),
            cls('ToddTheDinosaur', 'todd-the-dinosaur'),
            cls('WillyBlack', 'willy-black'),
            cls('WillyBlackSpanish', 'willy-black-spanish', lang='es'),
            cls('WorkingCats', 'working-cats'),
            cls('ZippyThePinhead', 'zippy-the-pinhead'),
            cls('Zits', 'zits'),
            cls('ZitsSpanish', 'jeremias', lang='es'),
            # END AUTOUPDATE
        )



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
from dosagelib.helpers import joinPathPartsNamer, queryNamer


class TestNamer:
    """
    Tests for comic namer.
    """

    def test_queryNamer(self):
        testurl = 'http://FOO?page=result&page2=result2'
        assert queryNamer('page')(self, testurl, "") == 'result'
        assert queryNamer('page2', True)(self, "", testurl) == 'result2'

    def test_joinPathPartsNamer(self):
        imgurl = 'https://HOST/wp-content/uploads/2019/02/tennis5wp-1.png'
        pageurl = 'https://HOST/2019/03/11/12450/'
        assert joinPathPartsNamer(pageparts=(0, 1, 2), imageparts=(-1,))(self,
            imgurl, pageurl) == '2019_03_11_tennis5wp-1.png'
        assert joinPathPartsNamer(pageparts=(0, 1, 2), imageparts=(-1,), joinchar='-')(self,
            imgurl, pageurl) == '2019-03-11-tennis5wp-1.png'
        assert joinPathPartsNamer(pageparts=(0, -1))(self, imgurl, pageurl) == '2019_12450'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from re import compile, escape

from .. import util
from ..helpers import indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import (
    ComicControlScraper,
    WordPressNavi,
    WordPressScraper,
    WordPressWebcomic,
)


class BackOffice(WordPressNavi):
    url = 'https://rawrtacular.com/bo/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % 'back-office'


class BadassMuthas(ParserScraper):
    url = ('https://web.archive.org/web/20200117233227/'
        'http://badassmuthas.com/pages/comic.php')
    stripUrl = url + '?%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src,"/comicsissue")]'
    prevSearch = '//a[./img[contains(@src,"/comicsbuttonBack")]]'
    help = 'Index format: nnn'


class BadMachinery(_ParserScraper):
    url = 'https://scarygoround.com/badmachinery/index.php'
    stripUrl = url + '?date=%s'
    firstStripUrl = stripUrl % '20090921'
    imageSearch = '//img[@class="comicimg"]'
    prevSearch = '//a[contains(text(), "Previous")]'
    endOfLife = True
    help = 'Index format: yyyymmdd'


class BalderDash(ComicControlScraper):
    url = 'http://www.balderdashcomic.com/'


class BallerinaMafia(_ParserScraper):
    url = 'https://web.archive.org/web/20200115230012/http://ballerinamafia.net/'
    stripUrl = url + 'index.php?pid=%s'
    firstStripUrl = stripUrl % '20100906'
    imageSearch = ('//img[contains(@alt, "Comic")]',
                   '//a[contains(@href, "comics/")]')
    prevSearch = '//a[@class="prev"]'
    adult = True
    endOfLife = True


class Bardsworth(WordPressScraper):
    url = 'http://www.bardsworth.com/'
    latestSearch = '//a[@rel="bookmark"]'
    starter = indirectStarter


class Baroquen(_BasicScraper):
    url = 'http://www.baroquencomics.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2008/11/05/raise-the-curtains'
    imageSearch = compile(tagre("img", "src", r'(%sComics/[^"]+)' % rurl))
    prevSearch = compile(tagre("a", "href", r'(%s[^"]+)' % rurl, after='prev'))
    help = 'Index format: yyyy/mm/dd/strip-name'


class BeachWzrd(WordPressScraper):
    url = 'https://mcnostril.com/comic/chapter-1-cover/'
    firstStripUrl = url
    imageSearch = '//div[@id="comic"]//noscript/img'
    latestSearch = '//a[d:class("comic-nav-last")]'
    starter = indirectStarter


class Bearmageddon(WordPressScraper):
    url = 'http://bearmageddon.com/bearmo/page-1/'
    firstStripUrl = url
    latestSearch = '//a[d:class("comic-nav-last")]'
    starter = indirectStarter


class Beetlebum(ParserScraper):
    url = 'https://blog.beetlebum.de/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2005/08/12/my-first-blog'
    starter = indirectStarter
    namer = joinPathPartsNamer(pageparts=range(-4, 0), imageparts=(-1,))
    multipleImagesPerStrip = True
    imageSearch = '//div[d:class("entry-content")]//img'
    prevSearch = '//a[@rel="prev"]'
    latestSearch = '//a[@rel="bookmark"]'
    help = 'Index format: yyyy/mm/dd/striptitle'
    lang = 'de'


class Bethellium(WordPressWebcomic):
    url = 'https://bethellium.darkbluecomics.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'cover'


class BetterDays(_ParserScraper):
    url = 'https://web.archive.org/web/20200201203404/http://jaynaylor.com/betterdays/'
    stripUrl = url + 'archives/%s.html'
    firstStripUrl = stripUrl % '2003/04/post-2'
    imageSearch = '//img[contains(@src, "/betterdays/comic/")]'
    prevSearch = '//a[contains(text(), "Previous")]'
    adult = True
    endOfLife = True


class BetweenFailures(WordPressWebcomic):
    url = 'https://betweenfailures.com/'
    stripUrl = url + 'comics1/%s'
    firstStripUrl = stripUrl % 'every-story-has-to-start-somewhere'
    help = 'Index format: stripname'


class BeyondTheVeil(WordPressScraper):
    url = 'https://web.archive.org/web/20201009235642/http://beyondtheveilcomic.com/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % '01252010'
    endOfLife = True

    def namer(self, image_url, page_url):
        # Fix inconsistent filenames
        filename = util.urlpathsplit(image_url)[-1]
        filename = filename.replace('BtV_pg43_bw', '2014-04-25-BtV_pg43_bw')
        filename = filename.replace('BtVpg28Ch7b', '2014-07-04-BtVpg28Ch7b')
        return filename


class BiggerThanCheeses(_BasicScraper):
    url = 'http://www.biggercheese.com/'
    stripUrl = url + 'index.php?comic=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(r'src="(comics/.+?)" alt')
    prevSearch = compile(r'"(index.php\?comic=.+?)".+?_back')
    help = 'Index format: n (unpadded)'


class BillyTheDunce(_ParserScraper):
    stripUrl = ('https://web.archive.org/web/20180404142544/'
        'http://www.duncepress.com/%s/')
    url = stripUrl % '2012/02/losing-more'
    firstStripUrl = stripUrl % '2009/06/an-introduction-of-sorts'
    imageSearch = '//div[@class="entry"]/p[1]/a'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True


class BirdBoy(WordPressScraper):
    url = 'https://bird-boy.com/'
    stripUrl = url + 'comic/{0}-{1}/'
    firstStripUrl = stripUrl.format('volume-i', 'the-sword-of-mali-mani')
    help = 'volume-page # Examples: i-the-sword-of-mali-mani, iii-7, synopsis-2'

    def getIndexStripUrl(self, index):
        (volume, strip) = index.split('-', maxsplit=1)
        try:
            pageNr = int(strip)
        except ValueError:
            pageNr = None  # Use the string to fetch a cover page
        if volume == 'synopsis':
            strip = '{0}{1}'.format(pageNr, '-02' if strip in [1, 3] else '')
        else:
            volume = 'volume-' + volume
            if pageNr is not None:
                strip = 'page-{0}'.format(pageNr)
        return self.stripUrl.format(volume, strip)

    def namer(self, image_url, page_url):
        # Fix inconsistent filenames
        imgparts = util.urlpathsplit(image_url)
        filename = imgparts[-1]
        if filename == 'image.jpg':
            year, month = imgparts[-3:-1]
            pageNr = int(page_url.rsplit('/', 1)[-2].rsplit('-', 1)[-1])
            filename = f'{year}-{month}-Vol2-pg{pageNr}.jpg'
        elif filename == '27637.jpg':
            filename = 'BB_Vol2_Cover.jpg'
        return filename


class BittersweetCandyBowl(ParserScraper):
    url = 'https://www.bittersweetcandybowl.com/'
    stripUrl = url + '%s.html'
    firstStripUrl = stripUrl % 'c1/p1'
    imageSearch = '//img[@id="page_img"]'
    prevSearch = '//a[@rel="prev"]'
    namer = joinPathPartsNamer(imageparts=range(-2, 0))


class BlankIt(_ParserScraper):
    url = 'http://blankitcomics.com/'
    firstStripUrl = url + 'comic/well-what-would-you-do'
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[d:class("comic-nav-previous")]'
    latestSearch = '//a[d:class("comic-nav-last")]'
    starter = indirectStarter


class BlondeSunrise(_ParserScraper):
    url = 'https://blondesunrise.com/comic.php?page=latest'
    firstStripUrl = 'https://blondesunrise.com/comic/comic.php?page=1'
    imageSearch = '//img[contains(@src, "comic_imgs/")]'
    prevSearch = '//a[img[contains(@src, "previous")]]'


class Bloodline(WordPressScraper):
    url = 'http://w0lfmare.xepher.net/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'pg-1-2'
    imageSearch = '//div[@id="comic"]//img[not(contains(@src, "TWC-vote-image"))]'

    def namer(self, image_url, page_url):
        # Fix filenames of early comics
        return util.urlpathsplit(image_url)[-1].replace('gen-6', 'Bloodline')


class BloomingFaeries(ParserScraper):
    url = 'https://www.bloomingfaeries.com/'
    firstStripUrl = url + 'comic/public/pit-stop/'
    imageSearch = '//div[@id="spliced-comic"]//img'
    prevSearch = '//a[d:class("previous-comic")]'
    adult = True
    namer = joinPathPartsNamer(imageparts=range(-3, 0))


class BMovieComic(_ParserScraper):
    url = 'https://bmoviecomic.com/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % 'chapter-1-strip-1'
    multipleImagesPerStrip = True
    imageSearch = '//div[d:class("entry-content")]//img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: pagename'


class BobWhite(_ParserScraper):
    url = 'http://www.bobwhitecomics.com/'
    imageSearch = '//span[d:class("webcomic-object")]/img'
    prevSearch = '//a[@rel="previous"]'


class BookOfBiff(WordPressScraper):
    url = 'https://thebookofbiff.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % '4'


class BoredAndEvil(_ParserScraper):
    url = 'http://orphanedcomics.com/boredandevil/'
    stripUrl = url + 'webcomic-%s.html'
    firstStripUrl = stripUrl % '2004-06-07'
    imageSearch = '//img[d:class("webcomic")]'
    prevSearch = '//a[img[@title="Previous"]]'
    endOfLife = True
    help = 'Index format: yyyy-mm-dd'


class BratHalla(WordPressScraper):
    url = 'http://brat-halla.com/'


class Brink(WordPressScraper):
    stripUrl = 'https://paperfangs.com/brink/?comic=%s'
    firstStripUrl = stripUrl % 'chapter1coversmall'
    url = stripUrl % 'brink639small'
    endOfLife = True


class Buni(WordPressScraper):
    url = 'http://www.bunicomic.com/'


class BusinessCat(_ParserScraper):
    url = 'https://www.businesscatcomic.com/'
    imageSearch = '//div[d:class("comic-image")]//img'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True


class ButImACatPerson(WordPressScraper):
    url = 'https://www.bicatperson.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = 'sketches-1'
    endOfLife = True


class ButtercupFestival(_ParserScraper):
    url = 'http://www.buttercupfestival.com/'
    stripUrl = url + '%s.htm'
    firstStripUrl = stripUrl % '2-1'
    imageSearch = '//center/img'
    prevSearch = (
        '//a[img[contains(@src, "previous")]]',  # 3-x
        '//a[text()="previous"]',  # 2-x
    )


class ButternutSquash(ParserScraper):
    url = ('https://web.archive.org/web/20170319023859/'
        'http://www.butternutsquash.net/')
    stripUrl = url + '2003/04/16/meet-da-punks/%s/'  # The website is very broken
    firstStripUrl = stripUrl % '2003/04/16/meet-da-punks'
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: yyyy/mm/dd/strip-name-author-name'


class ButterSafe(_ParserScraper):
    url = 'https://www.buttersafe.com/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2007/04/03/breakfast-sad-turtle'
    imageSearch = '//div[@id="comic"]/img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: yyyy/mm/dd/stripname'


class ByTheBook(WordPressScraper):
    url = 'http://www.btbcomic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'chapter-1-page-0'
    adult = True



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
import json
import logging
import os
import re

import pytest
import responses

import dosagelib
import httpmocks
from dosagelib import cmd, output


def run_cmd(*options):
    """'Fake' run dosage with given options."""
    retval = cmd.main(('--allow-multiple',) + options)
    # Cleanup logging
    root = logging.getLogger()
    ourhandlers = (x for x in root.handlers if isinstance(x, output.RichHandler))
    for handler in ourhandlers:
        root.removeHandler(handler)
    return retval


def cmd_ok(*options):
    assert run_cmd(*options) == 0


def cmd_err(*options):
    assert run_cmd(*options) == 1


@pytest.mark.usefixtures('_nosleep', '_noappdirs')
class TestDosage:
    """Test the dosage commandline client."""

    # This shouldn't hit the network at all, so add responses without mocks to
    # make sure it doesn't do that
    @responses.activate
    @pytest.mark.parametrize(('option'), [
        ('-l'),
        ('--list'),
        ('--singlelist'),
    ])
    def test_list_comics(self, option, capfd):
        cmd_ok(option)
        out = capfd.readouterr().out
        assert 'ADummyTestScraper' in out

    @responses.activate
    def test_display_version(self):
        cmd_ok("--version")

    @responses.activate
    def test_update_available(self, capfd):
        responses.add(responses.GET, re.compile(r'https://api\.github\.com/'),
            json={'tag_name': '9999.0', 'assets': [
                {'browser_download_url': 'TEST.whl'},
                {'browser_download_url': 'TEST.exe'},
            ]})
        cmd_ok('--version', '-v')
        out = capfd.readouterr().out
        best = 'TEST.exe' if os.name == 'nt' else 'TEST.whl'
        assert best in out
        assert 'A new version' in out

    @responses.activate
    def test_no_update_available(self, capfd):
        responses.add(responses.GET, re.compile(r'https://api\.github\.com/'),
            json={'tag_name': '1.0'})
        cmd_ok('--version', '-v')
        out = capfd.readouterr().out
        assert 'Detected local or development' in out

    @responses.activate
    def test_current(self, capfd):
        responses.add(responses.GET, re.compile(r'https://api\.github\.com/'),
            json={'tag_name': dosagelib.__version__})
        cmd_ok('--version', '-v')
        out = capfd.readouterr().out
        assert out.endswith('issues\n')

    @responses.activate
    def test_update_broken(self, capfd):
        responses.add(responses.GET, re.compile(r'https://api\.github\.com/'),
            json={})
        cmd_ok('--version', '-v')
        out = capfd.readouterr().out
        assert 'KeyError' in out

    @responses.activate
    def test_update_rate_limit(self, capfd):
        responses.add(responses.GET, re.compile(r'https://api\.github\.com/'),
            status=403)
        cmd_ok('--version', '-v')
        out = capfd.readouterr().out
        assert 'HTTPError' in out

    def test_display_help(self):
        for option in ("-h", "--help"):
            with pytest.raises(SystemExit):
                run_cmd(option)

    def test_module_help(self, capfd):
        cmd_ok("-m", "-t", "xkcd")
        out = capfd.readouterr().out
        assert re.match(r'([0-9][0-9]:){2}.. xkcd>', out)

    def test_broken_basepath_removal(self):
        assert run_cmd('-m', 'Comicsxkcd') == 2

    def test_working_basepath_removal(self):
        cmd_ok('-m', 'Comics/xkcd')
        cmd_ok('-m', 'Comics\\xkcd')

    def test_no_comics_specified(self):
        cmd_err()

    def test_unknown_option(self):
        with pytest.raises(SystemExit):
            run_cmd('--imadoofus')

    def test_multiple_comics_match(self):
        cmd_err('Garfield')

    @responses.activate
    def test_fetch_html_and_rss_json(self, tmpdir):
        httpmocks.xkcd()
        cmd_ok("-n", "2", "-v", "-b", str(tmpdir), "-o", "html", "-o", "rss",
               "-o", "json", "--no-downscale", "xkcd")

    @responses.activate
    def test_fetch_html_and_rss_2(self, tmp_path):
        httpmocks.page('https://www.bloomingfaeries.com/', 'bf-home')
        httpmocks.page(re.compile('.*/comic/601.*'), 'bf-601')
        httpmocks.png(re.compile(r'https://i0\.wp.*/BF601.*jpg'))
        httpmocks.png(re.compile(r'https://i0\.wp.*/BF602.*jpg'), 'tall')

        cmd_ok("--numstrips", "2", "--baseurl", "bla", "--basepath",
            str(tmp_path), "--output", "rss", "--output", "html", "--adult",
            "BloomingFaeries")

        html = next((tmp_path / 'html').glob('*.html')).read_text()
        assert "width=" in html

    @responses.activate
    def test_fetch_html_broken_img(self, tmp_path):
        httpmocks.page('https://www.bloomingfaeries.com/', 'bf-home')
        httpmocks.page(re.compile('.*/comic/601.*'), 'bf-601')
        responses.add(responses.GET, re.compile(r'.*\.jpg'), body=b'\377\330',
            content_type='image/jpeg')

        cmd_ok("--numstrips", "2", "--baseurl", "bla", "--basepath",
            str(tmp_path), "--output", "html", "--adult", "BloomingFaeries")

        html = next((tmp_path / 'html').glob('*.html')).read_text()
        assert "width=" not in html

    @responses.activate
    def test_fetch_indexed(self, tmpdir):
        httpmocks.xkcd()
        cmd_ok("-n", "2", "-v", "-b", str(tmpdir), "xkcd:303")

    @responses.activate
    def test_fetch_all_existing(self, tmp_path):
        httpmocks.xkcd()
        xkcd = tmp_path / 'xkcd'
        xkcd.mkdir()
        other = tmp_path / 'randomdir'
        other.mkdir()
        cmd_ok('-v', '-b', str(tmp_path), '@')
        assert len(list(xkcd.glob('*'))) == 2
        assert len(list(other.glob('*'))) == 0

    @responses.activate
    def test_json_page_key_bounce_and_multi_image(self, tmpdir):
        httpmocks.page(re.compile(r'.*com/$'), 'zp-home')
        httpmocks.page(re.compile(r'.*com/comic/missing/$'), 'zp-223')
        httpmocks.page(re.compile(r'.*com/comic/lifejacket/$'), 'zp-222')
        httpmocks.jpeg(re.compile(r'https://cdn-.*\.jpg'))

        cmd_ok("-v", "-b", str(tmpdir), "-o", "json", "ZenPencils")

        directory = tmpdir.join('ZenPencils')
        f = directory.join('dosage.json').open(encoding='utf-8')
        data = json.load(f)
        f.close()

        pages = data['pages']
        assert len(pages) == 1

        page = list(pages.keys())[0]
        assert page == 'https://zenpencils.com/comic/missing/'

        images = data['pages'][page]['images']
        assert len(images) == 2

        for imgfile in images.values():
            assert directory.join(imgfile).check(file=1)



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
"""
Script to get a list of KeenSpot comics and save the info in a
JSON file for further processing.
"""

from urllib import parse

from scriptutil import ComicListUpdater
from dosagelib import http


class KeenSpotUpdater(ComicListUpdater):
    dup_templates = ('Creators/%s', 'GoComics/%s', 'ComicGenesis/%s')

    # names of comics to exclude
    excluded_comics = (
        # non-standard navigation
        'BrawlInTheFamily',
        'Flipside',
        'LastBlood',
        'TheGodChild',
        'Twokinds',
        'Yirmumah',
    )

    extra = {
        'CrowScare': "last='20111031'",
        'Dreamless': "last='20100726'",
        'GeneCatlow': "last='20170412'",
        'MysticRevolution': "path='?cid=%s'",
        'PunchAnPie': "path='daily/%s.html'",
        'ShockwaveDarkside': "path='2d/%s.html'",
    }

    def collect_results(self):
        """Parse the front page."""
        data = self.get_url('http://keenspot.com/')

        for comiclink in data.xpath('//td[@id]/a'):
            comicurl = comiclink.attrib['href']
            name = comiclink.xpath('string()')
            try:
                if '/d/' not in comicurl:
                    http.check_robotstxt(comicurl + 'd/', self.session)
                else:
                    http.check_robotstxt(comicurl, self.session)
            except IOError as e:
                print('[%s] INFO: robots.txt denied: %s' % (name, e))
                continue

            self.add_comic(name, comicurl)

    def get_entry(self, name, url):
        sub = parse.urlsplit(url).hostname.split('.', 1)[0]
        if name in self.extra:
            extra = ', ' + self.extra[name]
        else:
            extra = ''
        return u"cls('%s', '%s'%s)," % (name, sub, extra)


if __name__ == '__main__':
    KeenSpotUpdater(__file__).run()



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
from ..scraper import ParserScraper


class Footloose(ParserScraper):
    url = 'http://footloosecomic.com/footloose.php'
    imageSearch = '//body/p[1]//img'
    prevSearch = '//body/a[2]'


class Cherry(Footloose):
    url = 'http://footloosecomic.com/cherry/index.php'


class Desigaspring(Footloose):
    url = 'http://footloosecomic.com/dspring/index.php'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
import codecs
import html
import json
import os
import re
import sys
import time

import lxml

from dosagelib import http, util, xml
from dosagelib.scraper import scrapers


def first_lower(x):
    return x[0].lower()


class ComicListUpdater:
    dup_templates: tuple[str, ...] = ()
    excluded_comics: tuple[str, ...] = ()

    START = "# START AUTOUPDATE"
    END = "# END AUTOUPDATE"

    def __init__(self, name: str):
        self.json = name.replace(".py", ".json")
        self.session = http.default_session
        self.sleep = 0

    def get_url(self, url: str, expand=True):
        """Get an HTML page and parse it with LXML."""
        print("Parsing", url, file=sys.stderr)
        try:
            pagetext = util.get_page(url, self.session).text
            data = lxml.html.document_fromstring(pagetext)
            if expand:
                data.make_links_absolute(url)
            if self.sleep > 0:
                time.sleep(self.sleep)
            return data
        except IOError as msg:
            print("ERROR:", msg, file=sys.stderr)
            raise

    def should_skip(self, name: str):
        if contains_case_insensitive(self.res, name):
            # we cannot handle two comics that only differ in case
            print("INFO: skipping possible duplicate", repr(name),
                  file=sys.stderr)
            return True
        return False

    def get_results(self):
        """Collect comics and save dictionary in JSON file."""
        self.res = {}
        self.collect_results()

        if not self.res:
            print("ERROR:", "did not match any comics", file=sys.stderr)
            return

        with codecs.open(self.json, 'wb', 'utf-8') as f:
            json.dump(self.res, f, sort_keys=True, indent=2,
                      separators=(',', ': '))

    def add_comic(self, name: str, data, count=None):
        """Add a collected comic with a specific number of comics."""
        name = format_name(name)
        if not self.should_skip(name):
            self.res[name] = {'count': count, 'data': data}
            return True
        return False

    def collect_results(self):
        raise NotImplementedError

    def print_results(self, args):
        """Print all comics that have at least the given number of minimum
        comic strips."""
        min_comics, filename = args
        min_comics = int(min_comics)
        oldf = codecs.open(filename, 'r', 'utf-8')
        newf = codecs.open(filename + '.new', 'w', 'utf-8')
        with oldf, newf:
            indent = self.copy_until_start(oldf, newf)
            with codecs.open(self.json, 'rb', 'utf-8') as f:
                data = json.load(f)
            for name, entry in sorted(data.items(), key=first_lower):
                self.write_entry(newf, name, entry, min_comics, indent)
            self.copy_after_end(oldf, newf)
        os.replace(filename + '.new', filename)

    def copy_until_start(self, src, dest):
        for line in src:
            dest.write(line)
            if line.strip().startswith(self.START):
                return line.find(self.START)
        raise RuntimeError("can't find start marker!")

    def copy_after_end(self, src, dest):
        skip = True
        for line in src:
            if line.strip().startswith(self.END):
                skip = False
            if not skip:
                dest.write(line)
        if skip:
            raise RuntimeError("can't find end marker!")

    def write_entry(self, fp, name, entry, min_comics, indent):
        if name in self.excluded_comics:
            return
        count = entry['count']
        if count and count < min_comics:
            return
        dup = self.find_dups(name)
        fp.write(" " * indent)
        if dup is not None:
            fp.write(u"# %s has a duplicate in %s\n" % (name, dup))
        else:
            fp.write(self.get_entry(
                truncate_name(name),
                entry['data']).replace("\n", "\n" + (" " * indent)) + "\n")

    def find_dups(self, name):
        """Check if comic name already exists."""
        names = [(tmpl % name).lower() for tmpl in self.dup_templates]
        if names:
            for scraper in scrapers.all():
                lname = scraper.name.lower()
                if lname in names:
                    return scraper.name
        return None

    def get_entry(self, name, data):
        """Return an entry for the module generator."""
        raise NotImplementedError

    def run(self):
        if len(sys.argv) > 1:
            self.print_results(sys.argv[1:])
        else:
            self.get_results()

    def xpath(self, data, path: str):
        return data.xpath(path, namespaces=xml.NS)


def contains_case_insensitive(adict, akey):
    """Check if key is in adict. The search is case insensitive."""
    for key in adict:
        if key.lower() == akey.lower():
            return True
    return False


def capfirst(text):
    """Uppercase the first character of text."""
    if not text:
        return text
    return text[0].upper() + text[1:]


def save_result(res, json_file):
    """Save result to file."""
    with codecs.open(json_file, 'wb', 'utf-8') as f:
        json.dump(res, f, sort_keys=True, indent=2, separators=(',', ': '))


def load_result(json_file):
    """Load contents of a json file."""
    with codecs.open(json_file, 'rb', 'utf-8') as f:
        return json.load(f)


def truncate_name(text):
    """Ensure the comic name does not exceed 50 characters."""
    return text[:50]


def asciify(name):
    """Remove non-ascii characters from string."""
    return re.sub("[^0-9a-zA-Z_]", "", name)


TRANS = str.maketrans({
    '&': 'And',
    '@': 'At',
    'ñ': 'n',
    'á': 'a',
    'ú': 'u',
})


def format_name(text):
    """Format a comic name."""
    name = html.unescape(text)
    name = "".join(capfirst(x) for x in name.split(" "))
    return asciify(name.translate(TRANS))



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: 2015 Tobias Gruetzmacher
"""
Script to get WebComicFactory comics and save the info in a JSON file for
further processing.
"""
import scriptutil


class WebComicFactoryUpdater(scriptutil.ComicListUpdater):

    def find_first(self, url):
        data = self.get_url(url)

        firstlinks = self.xpath(data, '//a[d:class("comic-nav-first")]')
        if not firstlinks:
            print("INFO:", "No first link on »%s«, already first page?" %
                  (url))
            return url
        return firstlinks[0].attrib['href']

    def collect_results(self):
        """Parse start page for supported comics."""
        url = 'http://www.thewebcomicfactory.com/'
        data = self.get_url(url)

        for comicdiv in self.xpath(data, '//div[d:class("ceo_thumbnail_widget")]'):
            comicname = self.xpath(comicdiv, './/h2')[0]
            comiclink = self.xpath(comicdiv, './/a')[0]
            comicurl = comiclink.attrib['href']
            name = comicname.text
            if 'comic-color-key' in comicurl:
                continue
            comicurl = self.find_first(comicurl)
            self.add_comic(name, comicurl)

    def get_entry(self, name, url):
        return (u"cls('%s',\n    '%s')," % (name, url))


if __name__ == '__main__':
    WebComicFactoryUpdater(__file__).run()



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2025 Tobias Gruetzmacher
import io
import logging
import threading
import traceback

from rich import console, table, text, theme

from . import logext

logger = logging.getLogger(__name__)


def setup_console() -> console.Console:
    return console.Console(theme=theme.Theme({
        # Simulates old dosage style
        "logging.level.error": "dim red",
        "logging.level.warning": "bold yellow",
        "logging.level.info": "default",
        "logging.level.debug": "white",
        "logging.level.trace": "dim",
    }))


def console_logging(console: console.Console, level: int, timestamps: bool) -> None:
    """
    Configure Python logging for simple Dosage console output. This tries to
    emulate Dosage's legacy output style as much as possible.

    Levels work roughly like this:
      0 - The default, with shortened exception logging
      1 - Enables MOREINFO and verbose exception logging
      2 - Enables DEBUG logging
      3 - Enables TRACE logging
    """
    root = logging.getLogger()
    root.setLevel(translate_level(level))
    handler = RichHandler(console, showtime=timestamps, stacktrace=level > 0)
    handler.setFormatter(logging.Formatter(datefmt="%X"))
    root.addHandler(handler)
    # We don't want to see the low-level requests logging
    logging.getLogger("urllib3").setLevel(logging.INFO)


def translate_level(level: int) -> int:
    return {
        0: logging.INFO,
        1: logext.MOREINFO,
        2: logging.DEBUG,
        3: logext.TRACE,
    }.get(level, logging.NOTSET)


class RichHandler(logging.Handler):
    '''Custom logging handler using rich for nice colors. This emulates the
    legacy dosage style, while taking some inspiration from rich's own
    logging handler.'''

    def __init__(self, console: console.Console, showtime: bool = False,
            stacktrace: bool = True) -> None:
        super().__init__()

        self.console = console
        self.showtime = showtime
        self.stacktrace = stacktrace
        self._lastts = text.Text("")

    def emit(self, record: logging.LogRecord) -> None:
        # "Hide" exception information from formatter
        exc_info = record.exc_info
        record.exc_info = None
        record.exc_text = None  # This is just a cache, so we don't need to restore it later

        message = self.format(record)
        msgstyle = f"logging.level.{record.levelname.lower()}"

        context = getattr(record, "context", '')
        if not context and threading.current_thread() != threading.main_thread():
            context = threading.current_thread().name

        output = table.Table.grid(padding=(0, 1))
        row: list = []
        if self.showtime:
            output.add_column(style="log.time")
            row.append(self.timetext(record))
        if context:
            output.add_column()
            row.append(text.Text(f"{context}>"))
        output.add_column(ratio=1, style="log.message", overflow="fold")

        msgtext = text.Text(style=msgstyle)
        if record.levelno > logging.INFO:
            msgtext.append(f"{record.levelname.upper()}: ")

        if getattr(record, "markup", False):
            msgtext.append_text(text.Text.from_markup(message))
        else:
            msgtext.append_text(text.Text(message))
        row.append(msgtext)

        output.add_row(*row)

        # Add styled exception data
        if self.stacktrace and exc_info:
            extrarow: list[console.RenderableType] = ([""] * (len(row) - 1))
            extrarow.append(text.Text.from_ansi(self.colorexception(exc_info)))
            output.add_row(*extrarow)

            # Restore for next handler
            record.exc_info = exc_info

        try:
            self.console.print(output)
        except Exception:
            self.handleError(record)

    def usesTime(self) -> bool:
        return self.showtime

    def timetext(self, record: logging.LogRecord) -> text.Text:
        '''
        Get formatted time text or an empty placeholder if the time would be
        the same as for the previous log message. This should only be called
        inside the lock taken by "emit".
        '''
        timestamp = text.Text(self.time(record))
        if self._lastts == timestamp:
            return text.Text(" " * len(timestamp))
        else:
            self._lastts = timestamp
            return timestamp

    def time(self, record: logging.LogRecord) -> str:
        if self.formatter:
            return self.formatter.formatTime(record, self.formatter.datefmt)
        else:
            return ""

    def colorexception(self, ei):
        """
        Format and return the specified exception information as a string with ANSI
        colors (on PYthon 3.13+).
        """
        sio = io.StringIO()
        traceback.print_exception(ei[0], ei[1], ei[2], limit=None, file=sio, colorize=True)
        s = sio.getvalue()
        sio.close()
        if s[-1:] == "\n":
            s = s[:-1]
        return s
